function [muu,prt,reliable]=mixed_mu_lb(M,blk,options)
%
% call:  [mulb,prt,reliable]=mixed_mu_lb(M,blk)
%        [mulb,prt,reliable]=mixed_mu_lb(M,blk,options)
%
% Computation of the (skew) mixed mu lower bound with a power algorithm.
%
% M is a complex matrix, blk describes the structure of the model
% perturbation Delta. blk must have 3 columns.
%
% M and Delta must be square.
%
% mulb is the value of the (skew) mu lower bound, prt is the destabilizing
% model perturbation. Ideally, det(I-prt*M)=0.
%
% options(1) = 1 --> use of the power algorithm by musyn if available
%                    if not, switch to SMT version of the power algorithm
%                    (only for classical mu problems)
%                    (options(2:5) is useless if options(1)=1)
%
% options(2) = 0 --> an initialization with an SVD method.
%            = 1 --> an initialization with the previous optimal point (if any)
%                    and then an initialisation with an SVD method.
%            = 2 --> first an initialization with the previous optimal point (if any)
%                    then, only if the power algorithm did not converge, an
%                    other initialisation with an SVD method.
%            = 3 --> Only an initialization with the previous optimal point (if any).
%                    (options(3) is useless if options(2)=3)
%
% options(3) = 0 --> 1 or 2 non-random initialisations (see above).
% options(3) > 0 --> number of additional random initializations.
% options(3) < 0 --> no additional random initialisation if the power algorithm converged
%                    with non randon-initialisations. Otherwise at most N additional random
%                    initialisations, where N =-options(3).
%
% options(4) = 0 --> the maximal number of iterations is computed inside the routine.
%            > 0 --> maximal number of iterations.
%
% options(5) = tolerance under which the power algorithm has converged.
% If options(5)=0 is chosen, the tolerance is fixed inside the routine.
%
% 'options' must be a 5 dimensional vector.
%
% defaut value: options = [0 0 0 0 0]
%
% Internal use of global variables B_SOL_MEM W_SOL_MEM QC_SOL_MEM BETAT_SOL_MEM BETAC_SOL_MEM
%
% reliable between 0 and 1: a measure of the reliability of the result.
%
% The power algorithm for classical mu was initially proposed in:
% P.M. Young and J.C. Doyle,
% "Computation of mu with real and complex uncertainties",
% Proc. of the CDC 1990, pp. 1230-1235.
%
% Skew Mu Toolbox - G. Ferreres and J.M. Biannic, Onera-DCSD Toulouse, France.
%
if nargin==2
  options=[];
  options(1)=0;
  options(2)=0;
  options(3)=0;
  options(4)=0;
  options(5)=0;
elseif nargin~=3
  error('MIXED_MU_LB: 2 or 3 input arguments');
end
%
% Preliminary tests
%
if length(options)~=5 error('MIXED_MU_LB: options must be a 5 dimensional vector'); end
if options(1),
    if exist('mu.m')~=2,
        options(1)=0;
        fprintf('Warning! mu.m is not in your path --> Switched to SMT version of power-algorithm \n');
    end;
end;
%    
if size(blk,2)~=3 error('MIXED_MU_LB: blk must have 3 columns'); end
%
if all(blk(:,3)==0) error('MIXED_MU_LB: problem with the 3th column of blk'); end
%
idx=find(blk(:,1)<0);
if ~isempty(idx)
  qqq=blk(idx,2);
  if any(qqq~=0)
    error('MIXED_MU_LB: full real blocks are not allowed');
  end
end
%
% The case of rectangular model perturbations is not accounted for.
%
if size(M,1)~=size(M,2) error('MIXED_MU_LB: M must be square'); end
%
if ~square_delta(blk) error('MIXED_MU_LB: Delta must be square'); end
%
nnn=sum(abs(blk(:,1)));
if size(M,1)~=nnn
  error('MIXED_MU_LB: wrong respective sizes of M and Delta');
end
%
if options(1)&any(blk(:,3)==0)
  error('MIXED_MU_LB: only classical mu problems with the musyn power algorithm');
end
%
if options(1)
  blk=blk(:,1:2);
  opt='Lw';
  [bnds,rowd,sens,rowp,rowg]=mu(M,blk,opt);
  muu=bnds(2);
  prt=unwrapp(rowp,blk);
  reliable=(muu~=0);
  return;
end
%
mr=0;
for i=1:size(blk,1)
  if blk(i,1)<0 mr=mr+1; end
end
%
% Default value of the tolerance
%
if options(5)==0 options(5)=1e-4; end
%
n=size(M,1);
%
% maximal number of iterations
%
n_blk=size(blk,1);
if options(4)<=0
  nb_iter_max=max(10*n_blk,50); % A tuning parameter
else
  nb_iter_max=options(4);
end
%
if any(blk(:,3)==0) nb_iter_max=2*nb_iter_max; end
%
global Indic_debug;
Indic_debug=0;
%
try_once=(options(2)==3);
%
if options(2)
  global B_SOL_MEM W_SOL_MEM QC_SOL_MEM BETAT_SOL_MEM BETAC_SOL_MEM;
  if length(B_SOL_MEM)~=n   options(2)=0; end
  if length(QC_SOL_MEM)~=mr options(2)=0; end
end
%
previous_optimal_point=(options(2)==2);
%
reliable=0;
muu=0;
prt=NaN*ones(size(M));
%
if try_once&(options(2)==0) return; end
%
idx=-1;
the_end=0;
while ~the_end
%
  if options(2)
    b=B_SOL_MEM;
    w=W_SOL_MEM;
    qc=QC_SOL_MEM;
    betac=BETAC_SOL_MEM;
    betat=BETAT_SOL_MEM;
    if Indic_debug fprintf('\n\n Initialisation with the previous optimal point \n\n'); end
    options(2)=0;
  elseif idx==-1
    idx=0;
    [u,s,v]=svd(M);
    b=v(:,1);
    b=b/norm(b);
    w=b;
    qc=2*(rand(mr,1)>0.5)-1;
    betac=1;
    betat=1;
    if Indic_debug fprintf('\n\n Initialisation with SVD \n\n'); end
  else
    if norm(imag(M))==0
      b=rand(n,1);
    else
      b=rand(n,1)+sqrt(-1)*rand(n,1);
    end
    b=b/norm(b);
    w=b;
    qc=2*(rand(mr,1)>0.5)-1;
    betac=1;
    betat=1;
    if Indic_debug fprintf('\n\n Random initialisation \n\n'); end
    idx=idx+1;
  end
%
  [beta,Q,b_sol,w_sol,qc_sol,betac_sol,betat_sol,nb_iter,indic,reliable_bis]=...
  mixed_mu_lb_2(M,blk,b,w,qc,betac,betat,nb_iter_max,options(5));
%
  if Indic_debug&indic fprintf('\n\n Pathological case: indic= %i\n\n',indic); pause_talk; end
%
  if beta>muu
    reliable=reliable_bis;
    prt=Q;
    muu=beta;
    B_SOL_MEM=b_sol;
    W_SOL_MEM=w_sol;
    QC_SOL_MEM=qc_sol;
    BETAC_SOL_MEM=betac_sol;
    BETAT_SOL_MEM=betat_sol;
  end
%
  if try_once
    the_end=1;
  elseif previous_optimal_point
    the_end=(muu~=0);
    previous_optimal_point=0;
  else
    if options(3)>=0
      idx_max=options(3);
      the_end=(idx>=idx_max);
    elseif (muu==0)
      idx_max=-options(3);
      the_end=(idx>=idx_max);   
    else
      the_end=1;
    end
  end
%
end
%
% ************************************************************************************
%
function [beta,Q,b_sol,w_sol,qc_sol,betac_sol,betat_sol,nb_iter,indic,reliable]=...
         mixed_mu_lb_2(M,blk,b,w,qc,betac,betat,nb_iter_max,tol)
%
% call : [beta,Q,b_sol,w_sol,qc_sol,betac_sol,betat_sol,nb_iter,indic,reliable]=...
%        mixed_mu_lb_2(M,blk,b,w,qc,betac,betat,nb_iter_max,tol)
%
% The main subroutine of mixed_mu_lb.m
%
% At the output, beta is a (skew) mu lower bound, Q a destabilizing model
% perturbation.
%
% M:   complex matrix.
% blk: structure of the model perturbation (with 3 columns).
%
% Power algorithm initialized with b,w,qc,betac,betat. Corresponding values at
% the output are b_sol,w_sol,qc_sol,betac_sol,betat_sol.
%
% nb_iter_max = maximal number of iterations. The effective number of
% iterations is nb_iter at the output.
%
% tol = tolerance under which the power algorithm has converged.
% Typical value: 1e-4.
%
% reliable between 0 and 1: a measure of the reliability of the result.
%
% Remark: if the global variable "Indic_debug" is set to 1,
% the internal behaviour of the algorithm appears on screen.
%
if nargin~=9 error('MIXED_MU_LB_2: 9 input arguments'); end
%
% toltol = minimal absolute value of some scalars, under which a pathological case 
% is considered to occur.
% The value of "indic" at the output indicates if such a case occured.
%
toltol=1e-15;
omega_nu=0;
%
N=sum(abs(blk(:,1)));
%
reliable=0;
beta=0;
Q=[];
b_sol=[];
w_sol=[];
qc_sol=[];
betac_sol=[];
betat_sol=[];
%
betat_f=betat;
betac_f=betac;
qt=qc;
%
if all(blk(:,1)>=0) % Case of a complex model perturbation
  alfac=[];
  alfat=[];
end
%
global Indic_debug;
if Indic_debug
  tab_betac=[];
  tab_betat=[];
  tab_qc=[];
  tab_qt=[];
  tab_ac=[];
  tab_at=[];
end
%
nb_iter=0;
indic=0;
the_end=0;
%
while ~the_end
%
% Iterative rescaling for skew mu problems
%
  betat_f=omega_nu*betat_f+(1-omega_nu)*betat; % use of the tuning parameter omega_nu
  qqq=mixed_mu_lb_4(blk,betat_f);
  MM=qqq*M;
%
% The following power algorithm is extracted from the paper by Young and
% Doyle (see the reference in the help of mixed_mu_lb.m), page 1234, equations
% (37) and (38)
%
% Computation of betat and a
%
  t=MM*b;
  betat=norm(t);
  if betat<toltol indic=1; return; end
  a=t/betat;
%
% Computation of qt
%
  i_real=1;
  for i=1:size(blk,1)
    if blk(i,1)<0
      b1=mixed_mu_lb_3(b,i,blk);
      a1=mixed_mu_lb_3(a,i,blk);
      w1=mixed_mu_lb_3(w,i,blk);
      tt=norm(a1);
      if tt<toltol indic=2; return; end
      alfat(i_real)=sign(qc(i_real))*norm(b1)/tt+real(a1'*w1);
      t=abs(alfat(i_real));
      if t>1 qqq=alfat(i_real)/t; else qqq=alfat(i_real); end
      qt(i_real)=qqq;
      i_real=i_real+1;
    end
  end
%
% Computation of z
%
  i_real=1;
  zz=[];
  for i=1:size(blk,1)
    w1=mixed_mu_lb_3(w,i,blk);
    a1=mixed_mu_lb_3(a,i,blk);
    if blk(i,1)<0        % real scalars
      z1=qt(i_real)*w1;
      i_real=i_real+1;
    elseif blk(i,2)==0   % complex scalars
      t=w1'*a1;       
      tt=norm(t);
      if tt<toltol indic=3; return; end
      z1=t/tt*w1;
    else                 % full complex blocks
      tt=norm(a1);
      if tt<toltol indic=4; return; end
      z1=norm(w1)/tt*a1;
    end
    zz=[zz;z1];
  end
  z=zz;
%
% Iterative rescaling for skew mu problems
%
  betac_f=omega_nu*betac_f+(1-omega_nu)*betac; % use of the tuning parameter omega_nu
  qqq=mixed_mu_lb_4(blk,betac_f);
  MM=qqq*M;
%
% Computation of betac and w
%
  betac_bis=betac;
  w_bis=w;
%
  t=MM'*z;                            
  betac=norm(t);
  if betac<toltol indic=5; return; end
  w=t/betac;  
%
% Computation of qc
%
  i_real=1;
  for i=1:size(blk,1)
    if blk(i,1)<0
      b1=mixed_mu_lb_3(b,i,blk);
      a1=mixed_mu_lb_3(a,i,blk);
      w1=mixed_mu_lb_3(w,i,blk);
      tt=norm(a1);
      if tt<toltol indic=6; return; end
      alfac(i_real)=sign(qt(i_real))*norm(b1)/tt+real(a1'*w1); % tuning parameter K
      t=abs(alfac(i_real));
      if t>1 qqq=alfac(i_real)/t; else qqq=alfac(i_real); end
      qc(i_real)=qqq;
      i_real=i_real+1;
    end
  end
%
% Computation of b
%
  i_real=1;
  bb=[];
  for i=1:size(blk,1)
    a1=mixed_mu_lb_3(a,i,blk);
    w1=mixed_mu_lb_3(w,i,blk);
    if blk(i,1)<0
      b1=qc(i_real)*a1;
      i_real=i_real+1;
    elseif blk(i,2)==0     
      t=a1'*w1;                          
      tt=norm(t);
      if tt<toltol indic=7; return; end
      b1=t/tt*a1;
    else
      tt=norm(w1);
      if tt<toltol indic=8; return; end
      b1=norm(a1)/tt*w1;
    end
    bb=[bb;b1];
  end
  b=bb;
%
  if Indic_debug
    tab_betat=[tab_betat betat];
    tab_betac=[tab_betac betac];
    tab_qc=[tab_qc qc];
    tab_qt=[tab_qt qt];
    tab_ac=[tab_ac alfac];
    tab_at=[tab_at alfat];
  end
%
  nb_iter=nb_iter+1;
  if nb_iter==1
    the_end_1=0;
    if Indic_debug
      fprintf('\n\n       betat     betac     a         b         z         w\n\n');
    end
  else
    ttt(1)=abs(betat-betat_old);
    ttt(2)=abs(betac-betac_old);
    ttt(3)=norm(a_old-a);
    ttt(4)=norm(b_old-b);
    ttt(5)=norm(z_old-z);
    ttt(6)=norm(w_old-w);
    if Indic_debug
      fprintf(' %2i : ',nb_iter);
      for i=1:6
        fprintf(' %3.2e ',ttt(i));
      end
      fprintf('\n');
    end
    the_end_1=all(ttt<=tol);
  end
  betat_old=betat;
  betac_old=betac;
  a_old=a;
  b_old=b;
  z_old=z;
  w_old=w;
  the_end=the_end_1|(nb_iter>nb_iter_max);  
%
end
%
% Computation of the destabilizing model perturbations
%
Q1=[];  % b = Q  a
Q2=[];  % z = Q* w
icpt=1;
for iii=1:size(blk,1)
  nnn=abs(blk(iii,1));
  idx=icpt:(icpt+nnn-1);
  bb=b(idx);
  aa=a(idx);
  zz=z(idx);
  ww=w_bis(idx);
  if blk(iii,2)==0      % scalar
    qqq1=diag(bb./aa);
    qqq2=diag(zz./ww);
    if blk(iii,1)<0 qqq1=real(qqq1); qqq2=real(qqq2); end
  else                  % full complex block
    qqq1=bb*aa';
    qqq1=qqq1/norm(qqq1);
    qqq2=zz*ww';
    qqq2=qqq2/norm(qqq2);
  end
  qqq2=qqq2';
  Q1=mdiag(Q1,qqq1);
  Q2=mdiag(Q2,qqq2);
  icpt=icpt+nnn;
end
%
if Indic_debug
  fprintf('\n betat = %6.3e, betac = %6.3e\n',betat,betac);
end
%
[Q1,beta1]=mixed_mu_lb_5(Q1,M,blk,betat);
[Q2,beta2]=mixed_mu_lb_5(Q2,M,blk,betac_bis);
%
if betat==0     reliable_1=0; else reliable_1=beta1/betat; end
if betac_bis==0 reliable_2=0; else reliable_2=beta2/betac_bis; end
%
if isempty(Q1)&isempty(Q2)
  Q=[];
  beta=0;
elseif isempty(Q1)
  Q=Q2;
  beta=beta2;
  reliable=reliable_2;
elseif isempty(Q2)
  Q=Q1;
  beta=beta1;
  reliable=reliable_1;
elseif beta1>beta2
  Q=Q1;
  beta=beta1;
  reliable=reliable_1;
else
  Q=Q2;
  beta=beta2;
  reliable=reliable_2;
end
%
if isempty(Q)
  b_sol=[];
  w_sol=[];
  qc_sol=[];
  betac_sol=[];
  betat_sol=[];
else
  b_sol=b;
  w_sol=w;
  qc_sol=qc;
  betac_sol=betac;
  betat_sol=betat;
end
%
if Indic_debug
  if ~isempty(Q)
    qqq=Q*M;
    qqq=eye(size(qqq))-qqq;
    fprintf('\n beta = %6.3e, condition number of I - Delta*M = %6.3e\n',beta,cond(qqq));
  else
    fprintf('\n No solution found\n');
  end
end
%
if Indic_debug
  idx=1:length(tab_betac);
  figure(1);
  clf;
  plot(idx,tab_betac,'g',idx,tab_betat,'m');
  grid on;
  title('betac (g) and betat (m)');
  if any(blk(:,1)<0)&0
    figure(2); clf;
    N=size(tab_qc,1); for i=1:N, subplot(N,1,i); plot(tab_qc(i,:)); grid on; end
    subplot(N,1,1); title('qc');
    figure(3); clf;
    N=size(tab_qt,1); for i=1:N, subplot(N,1,i); plot(tab_qt(i,:)); grid on; end
    subplot(N,1,1); title('qt');
    figure(4); clf;
    N=size(tab_at,1); for i=1:N, subplot(N,1,i); plot(tab_at(i,:)); grid on; end
    subplot(N,1,1); title('alfat');
    figure(5); clf;
    N=size(tab_ac,1); for i=1:N, subplot(N,1,i); plot(tab_ac(i,:)); grid on; end
    subplot(N,1,1); title('alfac');
  end
  figure(1);
  pause_talk;
end
%
% ************************************************************************************
%
function b=mixed_mu_lb_3(b,i,blk)
%
% a subroutine of mixed_mu_lb_2.m
%
if i==1
  idx=1:abs(blk(i,1));
else
  idx=1:(i-1);
  qqq=sum(abs(blk(idx,1)));
  idx=(qqq+1):(qqq+abs(blk(i,1)));
end
%
b=b(idx);
%
% ************************************************************************************
%
function  M=mixed_mu_lb_4(blk,alfa)
%
% a subroutine of mixed_mu_lb_2.m
%
M=[];
for i=1:size(blk,1)
  nnn=abs(blk(i,1));
  if blk(i,3)==0 M=[M;alfa*ones(nnn,1)]; else M=[M;ones(nnn,1)]; end
end
M=diag(M);
%
% ************************************************************************************
%
function [Q,beta]=mixed_mu_lb_5(Q,M,blk,beta)
%
% call:  [Q,beta]=mixed_mu_lb_5(Q,M,blk,beta)
%
% a subroutine of mixed_mu_lb_2.m
%
% sigma_max(Q) = 1 at the input, but not at the output of the routine.
%
if nargin~=4 error('MIXED_MU_LB_5: 4 input arguments'); end
%
if all(blk(:,3)==1)
%
% *** classical mu problem ***
%
  if all(blk(:,1)<0)
%
% Case of a real model perturbation
%
    e=eig(Q*M);
    qqq=[];
    for i=1:length(e)
      if abs(e(i))<=1e-16 qqq(i)=Inf; else qqq(i)=imag(e(i))/abs(e(i)); end
    end
    index=find(abs(qqq)<=1e-4); % a tuning parameter
    if isempty(index)
      beta=0;
      Q=[];
      return;
    else
      e=real(e(index));
      [qqq,i]=max(abs(e));
      Q=Q/e(i);
    end
%
  elseif all(blk(:,1)>0)
%
% Case of a complex model perturbation
%
    e=eig(Q*M);
    [qqq,i]=max(abs(e));
    Q=Q/e(i);
%
  else
%
% Case of a mixed model perturbation
%
% See P.M. Young, M.P. Newlin and J.C. Doyle
% "Computing bounds for the mixed mu problem"
% International Journal of Robust and Nonlinear Control
% pp 573-590, number 6, volume 5, 1995.
%
    idx1=[];
    idx2=[];
    icpt=1;
    for i=1:size(blk,1)
      nnn=abs(blk(i,1));
      idx=icpt:(icpt+nnn-1);
      if blk(i,1)<0 idx1=[idx1 idx]; else idx2=[idx2 idx]; end
      icpt=icpt+nnn;
    end
    M11=M(idx1,idx1);
    M12=M(idx1,idx2);
    M21=M(idx2,idx1);
    M22=M(idx2,idx2);
    Q1=Q(idx1,idx1)/beta; % real part of Delta
    Q2=Q(idx2,idx2);      % complex part of Delta
    qqq=M11*Q1;
    qqq=eye(size(qqq))-qqq;
    if cond(qqq)>=1e14
      Q2=zeros(size(Q2));
    else
      qqq=inv(qqq);
      qqq=M22+M21*Q1*qqq*M12;
      e=eig(Q2*qqq);
      [qqq,i]=max(abs(e));
      Q2=Q2/e(i);
    end
    Q=zeros(size(Q));
    Q(idx1,idx1)=Q1;
    Q(idx2,idx2)=Q2;
%
  end
%
  beta=1/norm(Q);
%
else
%
% *** skew mu problem ***
%
% Delta=diag(Delta1,Delta2)
%
% Delta1 is maintained inside its unit ball.
% The size of Delta2 is free.
%
  idx1=[];
  idx2=[];
  blk1=[];
  blk2=[];
  icpt=1;
  for i=1:size(blk,1)
    nnn=abs(blk(i,1));
    idx=icpt:(icpt+nnn-1);
    if blk(i,3)==0
      idx1=[idx1 idx];
      blk1=[blk1; blk(i,1:2)];
    else
      idx2=[idx2 idx];
      blk2=[blk2; blk(i,1:2)];
    end
    icpt=icpt+nnn;
  end
  M11=M(idx1,idx1);
  M12=M(idx1,idx2);
  M21=M(idx2,idx1);
  M22=M(idx2,idx2);
  Q1=Q(idx1,idx1);
  Q2=Q(idx2,idx2);
%
% Case of a complex model perturbation Delta2
%
  if all(blk2(:,1)>0)
%
    qqq=M11*Q1;
    qqq=eye(size(qqq))-qqq;
    if cond(qqq)>=1e14
      Q2=zeros(size(Q2));
    else
      qqq=inv(qqq);
      qqq=M22+M21*Q1*qqq*M12;
      e=eig(Q2*qqq);
      [qqq,i]=max(abs(e));
      Q2=Q2/e(i);
    end
    Q=zeros(size(Q));
    Q(idx1,idx1)=Q1;
    Q(idx2,idx2)=Q2;
%
  elseif any(blk2(:,1)>0)
%
% Case of a mixed model perturbation Delta2
%
    idx1=[];
    idx2=[];
    idx3=[];
    icpt=1;
    for i=1:size(blk,1)
      nnn=abs(blk(i,1));
      idx=icpt:(icpt+nnn-1);
      if blk(i,3)==0
        idx1=[idx1 idx];
      elseif blk(i,1)<0
        idx2=[idx2 idx];
      else
        idx3=[idx3 idx];
      end
      icpt=icpt+nnn;
    end
    Q1=mdiag(Q(idx1,idx1),Q(idx2,idx2)/beta);
    Q2=Q(idx3,idx3);
    idx1=[idx1 idx2];
    idx2=idx3;
    M11=M(idx1,idx1);
    M12=M(idx1,idx2);
    M21=M(idx2,idx1);
    M22=M(idx2,idx2);
    qqq=M11*Q1;
    qqq=eye(size(qqq))-qqq;
    if cond(qqq)>=1e14
      Q2=zeros(size(Q2));
    else
      qqq=inv(qqq);
      qqq=M22+M21*Q1*qqq*M12;
      e=eig(Q2*qqq);
      [qqq,i]=max(abs(e));
      Q2=Q2/e(i);
    end
    Q=zeros(size(Q));
    Q(idx1,idx1)=Q1;
    Q(idx2,idx2)=Q2;
%
  elseif any(blk1(:,1)>0)
%
% Delta2 is real, but Delta1 is complex or mixed
%
    idx1=[];
    idx2=[];
    idx3=[];
    icpt=1;
    for i=1:size(blk,1)
      nnn=abs(blk(i,1));
      idx=icpt:(icpt+nnn-1);
      if blk(i,3)==1
        idx3=[idx3 idx];
      elseif blk(i,1)<0
        idx1=[idx1 idx];
      else
        idx2=[idx2 idx];
      end
      icpt=icpt+nnn;
    end
    Q1=Q(idx1,idx1); % real part of Delta1 (which is maintained inside the unit ball)
    Q2=Q(idx2,idx2); % complex part of Delta1
    Q3=Q(idx3,idx3); % Delta2 (which is real)
%
    M11=M([idx1 idx3],[idx1 idx3]);
    M12=M([idx1 idx3],idx2);
    M21=M(idx2,[idx1 idx3]);
    M22=M(idx2,idx2);
    b_min=0.95*beta; % May be adjusted
    b_max=1.05*beta; % May be adjusted
    [beta_out_1,Qsol_1]=mixed_mu_lb_5_bis(b_min,Q,Q1,Q2,Q3,M11,M12,M21,M22,idx1,idx2,idx3);
    [beta_out_2,Qsol_2]=mixed_mu_lb_5_bis(b_max,Q,Q1,Q2,Q3,M11,M12,M21,M22,idx1,idx2,idx3);
    if (beta_out_1>1)&(beta_out_2<1)
       beta_max=b_min;
       beta_min=b_max;
       Qsol=Qsol_1;
    elseif (beta_out_1<1)&(beta_out_2>1)
       Qsol=Qsol_2;
       beta_max=b_max;
       beta_min=b_min;
    else
      beta=0;
      Q=[];
      return;
    end
    the_end=0;
    while ~the_end
      beta=(beta_min+beta_max)/2;
      [beta_out,QQ]=mixed_mu_lb_5_bis(beta,Q,Q1,Q2,Q3,M11,M12,M21,M22,idx1,idx2,idx3);
      if beta_out>1
        Qsol=QQ;
        beta_max=beta;
      else
        beta_min=beta;
      end
      qqq=(beta_max-beta_min)/max(beta_min,beta_max);
      qqq=abs(qqq);
      the_end=(qqq<=1e-3);       
    end
    Q=Qsol;
%
  else
%
% Delta1 and Delta2 are real
%
    nb_iter=1;
    nb_iter_max=50;
    beta_old=beta;
    MM=[M11 M12; M21 M22];
    the_end=0;
    while ~the_end
      qqq=mdiag(beta*Q1,Q2)*MM;
      e=eig(qqq);
      qqq=[];
      for i=1:length(e)
        if abs(e(i))<=1e-16 qqq(i)=Inf; else qqq(i)=imag(e(i))/abs(e(i)); end
      end
      index=find(abs(qqq)<=1e-4); % a tuning parameter
      if isempty(index)
        beta=0;
        Q=[];
        return;
      else
        e=real(e(index));
        [beta,i]=max(abs(e));
        Qsol=zeros(size(Q));
        Qsol(idx1,idx1)=Q1;
        Qsol(idx2,idx2)=Q2/e(i);
      end
      qqq=(beta_old-beta)/max(beta,beta_old);
      qqq=abs(qqq);
      beta_old=beta;
      nb_iter=nb_iter+1;
      if qqq<=1e-4 % a tuning parameter
        the_end=1;
      elseif nb_iter>nb_iter_max
        the_end=2;
      end       
    end
    if the_end==2
      Q=[];
      beta=0;
      return;
    else
      Q=Qsol;
    end
%
  end
%
  idx2=[];
  icpt=1;
  for i=1:size(blk,1)
    nnn=abs(blk(i,1));
    idx=icpt:(icpt+nnn-1);
    if blk(i,3)==1 idx2=[idx2 idx]; end
    icpt=icpt+nnn;
  end
  Q2=Q(idx2,idx2);
  beta=1/norm(Q2);
%
end
%
qqq=eye(size(M))-Q*M;
if cond(qqq)<1e4 Q=[]; beta=0; end % A very questionable threshold
%
% ************************************************************************************
%
function [beta_out,Qsol]=mixed_mu_lb_5_bis(beta,Q,Q1,Q2,Q3,M11,M12,M21,M22,idx1,idx2,idx3)
%
% a subroutine of mixed_mu_lb_5.m
%
QQ1=mdiag(Q1,Q3/beta);
qqq=M11*QQ1;
qqq=eye(size(qqq))-qqq;
if cond(qqq)>=1e14
  QQ2=zeros(size(Q2));
  beta_out=Inf;
else
  qqq=inv(qqq);
  qqq=M22+M21*QQ1*qqq*M12;
  e=eig(Q2*qqq);
  [beta_out,i]=max(abs(e));
  Q2=Q2/e(i);
end
Qsol=zeros(size(Q));
Qsol(idx1,idx1)=Q1;
Qsol(idx2,idx2)=Q2;
Qsol(idx3,idx3)=Q3/beta;
