function [tab_mu,tab_pert]=mixed_mu_lb_freq(sys,blk,puls,method,alfa,cxi)
%
% call:  [tab_mu,tab_pert]=mixed_mu_lb_freq(sys,blk,puls,method,alfa,cxi)
%
% Robustness analysis of the standard interconnection structure M(s) - Delta.
%
% Computation of a mixed (skew) mu lower bound at each point of a frequency gridding "puls".
%
% sys = system matrix (ss format) of M(s), a continuous-time plant.
%
% The structure of Delta is described by blk (musyn format).
% If blk has only 2 columns, a classical mu problem is considered.
% Otherwise the third column describes the potentially skew nature of the
% uncertainty.
%
% Delta and M(s) must be square.
%
% See mu_frequency_gridding.m for the meaning of alfa and cxi.
%
% Internal use of mixed_mu_lb.m with different options.
%
% method = 1 --> the fastest method (default value).
%          2 --> a slower, but potentially more accurate method.
%          3 --> an even slower, but potentially even more accurate method.
%
% N.B.: method = 2 or 3 seems to provide in many cases the same result as method = 1.
%
% Internal use of global variables B_SOL_MEM W_SOL_MEM QC_SOL_MEM BETAT_SOL_MEM BETAC_SOL_MEM.
%
% Skew Mu Toolbox - G. Ferreres and J.M. Biannic, Onera-DCSD Toulouse, France.
%
if nargin==3
  method=[];
  alfa=[];
  cxi=[];
elseif nargin==4
  alfa=[];
  cxi=[];
elseif nargin==5
  cxi=[];
elseif nargin~=6
  error('MIXED_MU_LB_FREQ: wrong number of input arguments');
end
%
% Default values of input arguments
%
if isempty(method) method=1; end
if isempty(alfa) alfa=0; end
if isempty(cxi) cxi=0; end
%
% Preliminary tests
%
idx=find(blk(:,1)<0);
if ~isempty(idx)
  qqq=blk(idx,2);
  if any(qqq~=0)
    error('MIXED_MU_LB_FREQ: full real blocks are not allowed');
  end
end
%
left_half_plane=(cxi==0)&(alfa==0);
if (~left_half_plane)&any(blk(:,1)>=0)
  error('MIXED_MU_LB_FREQ: complex uncertainties are undefined outside the imaginay axis');
end
%
if size(blk,2)==2
  blk=[blk ones(size(blk,1),1)];
elseif size(blk,2)~=3
  error('MIXED_MU_LB_FREQ: blk must have 2 or 3 columns');
end
%
[a,b,c,d,Ts]=ssdata(sys);
if Ts~=0
  error('MIXED_MU_LB_FREQ: sys must be continuous-time');
end
%
e=eig(a);
[w,z]=damp(a);
z=min(z);
if (max(real(e))>=alfa)|(z<=cxi)
  error('MIXED_MU_LB_FREQ: nominal closed loop poles are not inside the left half plane or truncated sector');
end
%
% The case of rectangular model perturbations is not accounted for.
%
if size(d,1)~=size(d,2)
  error('MIXED_MU_LB_FREQ: M(s) must be square');
end
%
if ~square_delta(blk)
  error('MIXED_MU_LB_FREQ: Delta must be square');
end
%
nnn=sum(abs(blk(:,1)));
if size(d,1)~=nnn
  error('MIXED_MU_LB_FREQ: wrong respective sizes of M(s) and Delta');
end
%
% Computation of the frequency response
%
rep_M=calc_freq_resp(sys,puls,alfa,cxi);
%
% A critical tuning parameter
%
tol=1e-4;
%
% options of mixed_mu_lb.m
%
if method==1
  options=[0 2 0 0 tol];
elseif method==2
  options=[0 1 0 0 tol];
else
  options=[0 1 -2 0 tol];
end
%
global B_SOL_MEM W_SOL_MEM QC_SOL_MEM BETAT_SOL_MEM BETAC_SOL_MEM;
B_SOL_MEM=[];
W_SOL_MEM=[];
QC_SOL_MEM=[];
BETAT_SOL_MEM=[];
BETAC_SOL_MEM=[];
%
tab_B_SOL_MEM=[];
tab_W_SOL_MEM=[];
tab_QC_SOL_MEM=[];
tab_BETAT_SOL_MEM=[];
tab_BETAC_SOL_MEM=[];
%
nw=length(puls);
%
fprintf('\n%i: ',nw);
tab_reliable=[];
tab_mu=[];
tab_pert=[];
for iw=1:nw
  fprintf('%i',iw);
  M=rep_M(:,:,iw);  
  [muu,prt,reliable]=mixed_mu_lb(M,blk,options);
  OK=(muu~=0)&(reliable>=0.9);     % The threshold may be adjusted
  tab_reliable=[tab_reliable OK];
  tab_mu(iw)=muu;
  tab_pert(:,:,iw)=prt;
  tab_B_SOL_MEM{iw}=B_SOL_MEM;
  tab_W_SOL_MEM{iw}=W_SOL_MEM;
  tab_QC_SOL_MEM{iw}=QC_SOL_MEM;
  tab_BETAT_SOL_MEM{iw}=BETAT_SOL_MEM;
  tab_BETAC_SOL_MEM{iw}=BETAC_SOL_MEM;
  if OK fprintf('*'); else fprintf('.'); end
end
%
fprintf('\n%i: ',nw);
options=[0 3 0 0 tol]; % options of mixed_mu_lb.m
for iw=(nw-1):(-1):1
  if tab_reliable(iw)==0
    fprintf('%i',iw);
    M=rep_M(:,:,iw);
    B_SOL_MEM=tab_B_SOL_MEM{iw+1};
    W_SOL_MEM=tab_W_SOL_MEM{iw+1};
    QC_SOL_MEM=tab_QC_SOL_MEM{iw+1};
    BETAT_SOL_MEM=tab_BETAT_SOL_MEM{iw+1};
    BETAC_SOL_MEM=tab_BETAC_SOL_MEM{iw+1};
    [muu,prt]=mixed_mu_lb(M,blk,options);
    if muu>tab_mu(iw)
      tab_mu(iw)=muu;
      tab_pert(:,:,iw)=prt;
      tab_B_SOL_MEM{iw}=B_SOL_MEM;
      tab_W_SOL_MEM{iw}=W_SOL_MEM;
      tab_QC_SOL_MEM{iw}=QC_SOL_MEM;
      tab_BETAT_SOL_MEM{iw}=BETAT_SOL_MEM;
      tab_BETAC_SOL_MEM{iw}=BETAC_SOL_MEM;
      fprintf('*');
    else
      fprintf('.');      
    end
  end
end
fprintf('\n');
