function bnds=mixed_mu_ub(M,blk,options)
%
% call:  muub=mixed_mu_ub(M,blk)
%        muub=mixed_mu_ub(M,blk,options)
%
% Computation of the mixed (skew) mu upper bound.
%
% M is a complex matrix, blk describes the structure of the model
% perturbation. If blk has only 2 columns, a classical mu problem
% is considered.
%
% options = 0 --> use of the (skew) mu algorithm of the toolbox.
%           1 --> use of mu.m (if musyn available - if not switch to option 0)
%           (only for classical mu problems)
%           2 --> use of mubnd.m (LMI Control Toolbox)
%           (only for classical mu problems)
%
% Default value: options = 0.
%
% This routine essentially calls mu_ub.m or skew_mu_ub.m
% (if options==0)
%
% For the classical mu upper bound see:
% M.K.H. Fan and A.L. Tits and J.C. Doyle,
% "Robustness in the presence of mixed parametric uncertainty and unmodeled dynamics",
% IEEE Transactions on Automatic Control, vol. 36, n. 1, pp. 25-38, 1991.
%
% For the skew mu one see:
% G. Ferreres and V. Fromion,
% "Computation of the robustness margin with the skewed mu tool",
% Systems and Control Letters,
% Vol. 32, n. 4, pp. 193-202, December 1997.
%
% Skew Mu Toolbox - G. Ferreres and J.M. Biannic, Onera-DCSD Toulouse, France.
%
if nargin==2
  options=0;
elseif nargin~=3
  error('MIXED_MU_UB: 2 or 3 input arguments');
end
%
if options==1,
  if exist('mu.m') ~= 2,
     fprintf('Warning! mu.m is not in your path  --> Switched to mu_ub.m \n');
     options=0;
  end;
end;
if size(blk,2)==2
  classical_mu_problem=1;
elseif all(blk(:,3)==1)
  classical_mu_problem=1;
else
  classical_mu_problem=0;
  if options
    error('MIXED_MU_UB: only classical mu problems with mu.m or mubnd.m');
  end
end
%
if classical_mu_problem
  blk=blk(:,1:2);
  if options==0
    bnds=mu_ub(M,blk,1e-2,1,1e-3);
  elseif options==1
    opt='Uw';
    bnds=mu(M,blk,opt);
    bnds=bnds(1);
  else
    blk_lmi=calc_blk(blk);
    bnds=mubnd(M,blk_lmi,1e-3);
  end
else
  bnds=skew_mu_ub(M,blk,1e-2,1,1e-3);
end
