function [muu,pert]=mu_dailey(H,blk)
%
% call:  [mulb,pert]=mu_dailey(M,blk)
%
% Computation of a mu lower bound for a complex matrix M and
% a model perturbation Delta, whose structure is described by blk. Delta
% can only contain non-repeated real uncertainties. The method is
% exponential-time, and thus limited to about 10 or 12 uncertain
% parameters.
%
% The third column of blk describes the potentially skew or 
% one-sided skew nature of the uncertainty: 
%
% blk(i,3) = 0 --> uncertain parameter # i maintained inside [-1,1]
%          = 1 --> uncertain parameter # i inside [-k,k], where k is maximized.
%          = 2 --> uncertain parameter # i inside [0,k],  where k is maximized.
%
% See for further details:
% R.L. Dailey,
% "A new algorithm for the real structured singular value",
% Proc. of the ACC 1990, pp. 3036-3040.
%
% Skew Mu Toolbox - G. Ferreres and J.M. Biannic, Onera-DCSD Toulouse, France.
%
if nargin~=2 error('MU_DAILEY: two inputs arguments'); end
%
if any(blk(:,1)~=-1)|any(blk(:,2)~=0)
  error('MU_DAILEY: only non-repeated real uncertainties are allowed');
end
%
% tol = relative tolerance for the dichotomy search
%
% If the mu lower bound is less than mu_min, the mu lower bound is set to 0
%
tol=5e-2;
mu_min=1e-3;
%
Kmax=1/mu_min;
%
% The method does not work for the special case of a real matrix H
%
qqq=norm(imag(H))/norm(H);
if qqq<=1e-6
  muu=0;
  pert=NaN*ones(1,size(H,1));
  pert=diag(pert);
  return;
end
%
TAB=hypercube(size(H,1)-2);
%
% Preliminary test: is the mu lower bound less than mu_min ?
%
% mu_Dailey_bis.m checks whether a face exists, which contains a destabilizing 
% model perturbation with norm Kmax. If such a face exists indic is set to 1.
%
[indic,D]=mu_Dailey_bis(TAB,H,Kmax,blk);
%
if ~indic
  muu=0;
  pert=NaN*ones(1,size(H,1));
  pert=diag(pert);
  return;
else
  K1=0;
  K2=Kmax;
  D2=D;
end
%
% Dichotomy search
%
the_end=0;
while ~the_end
  K=(K1+K2)/2;
  [indic,D]=mu_Dailey_bis(TAB,H,K,blk);
  if indic
    K2=K;
    D2=D;
  else
   K1=K;
  end
  gap=(K2-K1)/K2;
  the_end=(gap<=tol);
end
%
muu=1/K2;
pert=D2;
pert=diag(pert);
%
% *****************************************************************
%
function [indic,D]=mu_Dailey_bis(TAB,H,K,blk)
%
% call:  [indic,D]=mu_Dailey_bis(TAB,H,K,blk)
%
% November 02
%
if nargin~=4 error('MU_DAILEY_BIS: 4 input arguments'); end
%
blk=blk(:,3);
%
N=size(H,1);
%
indic=0;
for i=1:N
  for j=(i+1):N
    idx1=[i j];
    idx2=[];
    for iii=1:N
      if (iii~=i)&(iii~=j) idx2=[idx2 iii]; end
    end
    M11=H(idx1,idx1);
    M12=H(idx1,idx2);
    M21=H(idx2,idx1);
    M22=H(idx2,idx2);
    blk1=blk(idx1);
    blk2=blk(idx2);
    for k=1:size(TAB,1)
%
      D2=TAB(k,:);
      for iii=1:length(D2)
        if blk2(iii)==1
          D2(iii)=K*D2(iii);
        elseif blk2(iii)==2
          if D2(iii)==-1
            D2(iii)=0;
          else
            D2(iii)=K;
          end
        end
      end
%
      D2=diag(D2);
      A=eye(size(D2))-D2*M22;
      q=det(A);
      if q==0   % if A is singular, matrix I-diag(D)*M is singular for all D1
        indic=1;
        D1=[0 0];
        D2=diag(D2);
        D=ones(1,size(H,1));
        D(i)=D1(1);
        D(j)=D1(2);
        icpt=1;
        for iii=1:size(H,1)
          if (iii~=i)&(iii~=j)
            D(iii)=D2(icpt);
            icpt=icpt+1;
          end
        end
        return;
      end
      A=M11+M12*inv(A)*D2*M21;
%
      d1=[];
      d2=[];
%
% [d1 d2] = solutions of the second order equation
%
      c1r=real(A(1,1));
      c1i=imag(A(1,1));
      c2r=real(A(2,2));
      c2i=imag(A(2,2));
      c12=A(1,1)*A(2,2)-A(1,2)*A(2,1);
      c12r=real(c12);
      c12i=imag(c12);
      a=c12i*c1r-c12r*c1i;
      b=-c12i+c1i*c2r-c1r*c2i;
      c=c2i;
%
      qqq=max(abs([a b c]));
      if qqq>1e-6
        a=a/qqq;
        b=b/qqq;
        c=c/qqq;
        if abs(a)>1e-6
          d=b*b-4*a*c;
          if d>=0
            d=sqrt(d);
            d1(1)=(-b+d)/(2*a);
            d1(2)=(-b-d)/(2*a);
            d2(1)=-c1i*d1(1)/(c2i-c12i*d1(1));
            d2(2)=-c1i*d1(2)/(c2i-c12i*d1(2));
          end
        elseif abs(b)>1e-6
          d1=-c/b;
          d2=-c1i*d1/(c2i-c12i*d1);
        end
      end
%
% A satisfactory model perturbation has been found ?
%
      for iii=1:length(d1)
        if blk1(1)==0
          OK_d1(iii)=(abs(d1(iii))<=1);
        elseif blk1(1)==1
          OK_d1(iii)=(abs(d1(iii))<=K);
        else
          OK_d1(iii)=(d1(iii)>=0)&(d1(iii)<=K);
        end
      end
      for iii=1:length(d2)
        if blk1(2)==0
          OK_d2(iii)=(abs(d2(iii))<=1);
        elseif blk1(2)==1
          OK_d2(iii)=(abs(d2(iii))<=K);
        else
          OK_d2(iii)=(d2(iii)>=0)&(d2(iii)<=K);
        end
      end
%
      if isempty(d1)
        indic=0;
        D=[];
      elseif length(d1)==1
        if OK_d1(1)&OK_d2(1)
          indic=1;
          D1=[d1 d2];
        else
          indic=0;
          D=[];
        end
      else
        if OK_d1(1)&OK_d2(1)
          indic=1;
          D1=[d1(1) d2(1)];
        elseif OK_d1(2)&OK_d2(2)
          indic=1;
          D1=[d1(2) d2(2)];
        else
          indic=0;
          D=[];
        end
      end
%
      if indic
        D2=diag(D2);
        D=ones(1,size(H,1));
        D(i)=D1(1);
        D(j)=D1(2);
        icpt=1;
        for iii=1:size(H,1)
          if (iii~=i)&(iii~=j)
             D(iii)=D2(icpt);
             icpt=icpt+1;
          end
        end
        return;
      end
%
    end
%
% End loop on k 
%
  end
%
% End loop on j 
%
end
%
% End loop on i
%
