function [list_intervals,list_intervals_OK,critical_pts,tab_mu,list_intervals_bis]=...
         mu_elim(sys,blk,gamma,list_intervals,nbr_iter_max,opt,possible_stop,epsilon,verbatim)
%
% call:  [list_intervals,list_intervals_OK,critical_pts,tab_mu,list_intervals_bis]=...
%        mu_elim(sys,blk,gamma,list_intervals,nbr_iter_max,opt,possible_stop,epsilon,verbatim)
%
% Robustness analysis of the interconnection structure M(s) - Delta.
%
% sys = system matrix of M(s) (ss format)
%
% blk describes the structure of the model perturbation Delta.
% blk must have three columns.
%
% M(s) and Delta must be square.
%
% Elimination of frequency intervals inside a specified frequency interval [wmin, wmax],
% which satisfy (skew) mu_ub(M(jw)) <= gamma.
%
% To this aim, [wmin, wmax] is partitioned at each iteration into smaller intervals.
% nbr_iter_max represents the maximal number of iterations.
%
% If a skew mu problem is considered, the problem is reduced to a
% classical mu test: mu_ub(N(jw)) <= 1.
%
% The mu upper bound mu_ub(M(jw)) or mu_ub(N(jw)) is computed with mu.m.
% opt is the option of mu.m (e.g. opt = 'f' or opt = 'u').
% 'w' will automatically be added. See the help of mu.m for more details.
%
% epsilon is used to regularize the mu problem, in order to render the scalings suboptimal.
%
% The input argument list_intervals is typically [wmin wmax],
% but it can also be the output argument list_intervals_bis (see below).
%
% If possible_stop is non-zero, the algorithm stops if a frequency w is found,
% for which (skew) mu_ub(M(jw)) >= gamma.
%
% There are two main situations:
% 1/ [wmin wmax] is a large frequency interval: nbr_iter_max is low (about 5)
%    and possible_stop=0. The issue is to eliminate frequency intervals
%    inside [wmin wmax], which satisfy (skew) mu_ub(M(jw)) <= gamma.
% 2/ [wmin wmax] is a small frequency interval: nbr_iter_max can be higher
%    but possible_stop=1. The issue is to check if (skew) mu_ub(M(jw)) <= gamma
%    on [wmin,wmax].
%
% verbatim = 0 --> minimal display
%          = 1 --> display of intermediate results
%          = 2 --> display of intermediate results and pause at the end
%                  of each iteration
%
% At the output, list_intervals contains frequency intervals,
% which do not satisy (skew) mu_ub(M(jw)) <= gamma. list_intervals is sorted and compacted.
% list_intervals = [] means that all frequencies were eliminated.
%
% Conversely, list_intervals_OK contains frequency intervals,
% which satisy (skew) mu_ub(M(jw)) <= gamma. list_intervals_OK is sorted and compacted.
%
% critical_pts contains frequencies where (skew) mu_ub(M(jw)) >= gamma
% (with opt used as the option for mu.m). tab_mu contains corresponding values of the
% mu lower and upper bounds.
%
% CAUTION ! If a skew mu problem is considered tab_mu does not correspond to
% skew mu upper and lower bounds !!!
%
% list_intervals_bis is not sorted, nore compacted. It is possible to use it
% as an input argument:
%
% [list_intervals,list_intervals_OK,critical_pts,list_intervals_bis]=...
% mu_elim_freq(sys,blk,gamma,[wmin wmax],nbr_iter_max,opt,possible_stop,epsilon);
%
% [list_intervals,list_intervals_OK,critical_pts,list_intervals_bis]=...
% mu_elim_freq(sys,blk,gamma,list_intervals_bis,1,opt,possible_stop,epsilon);
%
% The second call enables to further eliminate frequencies.
%
% Skew Mu Toolbox - G. Ferreres and J.M. Biannic, Onera-DCSD Toulouse, France.
%
if nargin~=9 error('MU_ELIM: 9 input arguments'); end
%
% Preliminary tests
%
idx=find(blk(:,1)<0);
if ~isempty(idx)
  qqq=blk(idx,2);
  if any(qqq~=0)
    error('MU_ELIM: full real blocks are not allowed');
  end
end
%
e=eig(sys);
if max(real(e))>=0
  error('MU_ELIM: M(s) is not asymptotically stable');
end
%
% The case of rectangular model perturbations is not accounted for.
%
if size(sys,1)~=size(sys,2)
  error('MU_ELIM: M(s) must be square');
end
%
if ~square_delta(blk)
  error('MU_ELIM: Delta must be square');
end
%
nnn=sum(abs(blk(:,1)));
if size(sys,1)~=nnn
  error('MU_ELIM: wrong respective sizes of M(s) and Delta');
end
%
if size(blk,2)~=3
  error('MU_ELIM: blk must have 3 columns');
end
%
classical_mu=all(blk(:,3)==1);
if ~classical_mu
  T=[];
  for i=1:size(blk,1)
    nnn=abs(blk(i,1));
    qqq=ones(1,nnn);
    if blk(i,3) qqq=qqq/gamma; end
    T=[T qqq];
  end
  T=diag(T);
  [a,b,c,d]=ssdata(sys);
  c=T*c;
  d=T*d;
  sys=ss(a,b,c,d);
  gamma=1;
end
%
omega_min=min(list_intervals(:));
omega_max=max(list_intervals(:));
%
blk=blk(:,1:2);
%
list_intervals=[list_intervals ones(size(list_intervals,1),1)];
%
stop_required=0;
nbr_iter=1;
critical_pts=[];
tab_mu=[];
the_end=0;
while ~the_end
  for i=1:size(list_intervals,1)
    if list_intervals(i,3)
      w1=list_intervals(i,1);
      w2=list_intervals(i,2);
      w0=(w1+w2)/2;
      M=freqresp(sys,w0);
      M=squeeze(M);
      if epsilon
        MM=[M sqrt(epsilon)*M; sqrt(epsilon)*M epsilon*M];
        blkk=[blk; size(M,1) size(M,1)];
        qqq=[opt,'Uw']; % it's useless to compute a mu lower bound, see below
      else
        MM=M;
        blkk=blk;
        qqq=[opt,'w'];
      end
      [bnds,rowd,sens,rowp,rowg]=mu(MM,blkk,qqq);
% A mu lower bound for the regularized mu problem is not a lower bound for the original one.
      if epsilon~=0 bnds(2)=0; end
      muu=bnds(1);
      [dl,dr,gl,gm,gr]=muunwrap(rowd,rowg,blkk);
      D=dl;
      G=gr;
      if isempty(G) G=zeros(size(D)); end
      nnn=size(M,1);
      D=D(1:nnn,1:nnn);
      G=G(1:nnn,1:nnn);
      if verbatim
        fprintf('\n Frequency interval between %8.5f and %8.5f rad/s: mu/gamma = %6.3f at %8.5f rad/s\n',...
        w1,w2,muu/gamma,w0);
      end
      if muu>=gamma
        list_intervals=[list_intervals; w1 w0 1; w0 w2 1];
        critical_pts=[critical_pts w0];
        tab_mu=[tab_mu; bnds];
        list_intervals(i,3)=0;
        if possible_stop stop_required=1; break; end
      else
        qqq1=D*M*inv(D)/gamma-sqrt(-1)*G;
        qqq2=eye(size(G))+G*G;
        qqq2=qqq2^(-1/4);
        qqq=qqq2*qqq1*qqq2;
        if norm(qqq)>1
          fprintf('\n\n *** MU_ELIM: problem with mu.m ? ***\n\n');
          list_intervals=[list_intervals; w1 w0 1; w0 w2 1];
          list_intervals(i,3)=0;
        else
	  warning off;
          [wmin,wmax]=calc_interval_bis(sys,w0,D,G,gamma,1);
	  warning on;
          list_intervals=mu_elim_ter(wmin,wmax,list_intervals);
          if verbatim
            fprintf('\n scalings valid between %8.5f and %8.5f rad/s\n',wmin,wmax);
          end
        end
      end
    end
  end
  list_intervals_new=[];
  for i=1:size(list_intervals,1)
    if list_intervals(i,3)
      list_intervals_new=[list_intervals_new; list_intervals(i,1) list_intervals(i,2) 1];
    end
  end 
  list_intervals=list_intervals_new;
  if verbatim
    if ~isempty(list_intervals)
      fprintf('\n Frequency intervals to be studied at the end of iteration %i:\n',nbr_iter);
      disp(list_intervals(:,1:2));
    else
      fprintf('\n All frequency intervals were eliminated at the end of iteration %i !\n',nbr_iter);
    end
    if verbatim==2 pause_talk; end
  end
  if stop_required
    the_end=1;
  else
    nbr_iter=nbr_iter+1;
    if isempty(list_intervals)
      the_end=1;
    else
      the_end=(nbr_iter>nbr_iter_max);
    end
  end
end
%
if isempty(list_intervals)
  list_intervals_bis=[];
  list_intervals_OK=[omega_min omega_max];
  critical_pts=[];
  tab_mu=[];
else
  list_intervals=list_intervals(:,1:2);
  list_intervals_bis=list_intervals;
  if size(list_intervals,1)==1
    w1=list_intervals(1,1);
    w2=list_intervals(1,2);
    list_intervals_OK=[omega_min w1; w2 omega_max];
  else
    list_intervals=mu_elim_bis(list_intervals);
    list_intervals_OK=[];
    w0=omega_min;
    for i=1:size(list_intervals,1)
      w1=list_intervals(i,1);
      w2=list_intervals(i,2);
      list_intervals_OK=[list_intervals_OK; w0 w1];
      w0=w2;
    end
    list_intervals_OK=[list_intervals_OK; w0 omega_max];
  end
  if omega_min==list_intervals_OK(1,2)
    list_intervals_OK=list_intervals_OK(2:end,:);
  end
  if omega_max==list_intervals_OK(end,1)
    list_intervals_OK=list_intervals_OK(1:(end-1),:);
  end
  critical_pts_new=[];
  tab_mu_new=[];
  for i=1:length(critical_pts)
    OK=1;
    w=critical_pts(i);
    for j=1:size(list_intervals_OK,1)
      w1=list_intervals_OK(j,1);
      w2=list_intervals_OK(j,2);
      if (w>=w1)&(w<=w2) OK=0; break; end
    end
    if OK
      critical_pts_new=[critical_pts_new w];
      tab_mu_new=[tab_mu_new; tab_mu(i,:)];
    end
  end
  critical_pts=critical_pts_new;
  tab_mu=tab_mu_new;
  if ~isempty(critical_pts)
    [critical_pts,index]=sort(critical_pts);
    tab_mu=tab_mu(index,:);
  end
end
%
% ****************************************************************************
%
function list=mu_elim_bis(list1,list2)
%
% call:  list=mu_elim_bis(list1,list2)
%
% a subroutine of mu_elim.m
%
if nargin==1
  list2=[];
elseif nargin~=2
  error('MU_ELIM_BIS: 1 or 2 input arguments');
end
%
list=[list1; list2];
%
if size(list,1)==1 return; end
%
[qqq,index]=sort(list(:,1));
list=list(index,:);
list=[list ones(size(list,1),1)];
list_new=[];
for i=1:size(list,1)
  if list(i,3)
    w1=list(i,1);
    w2=list(i,2);
    for j=1:size(list,1)
      if list(j,3)&(i~=j)
        w3=list(j,1);
        w4=list(j,2);
        if (w3>=w1)&(w4<=w2)
          list(j,3)=0;
        elseif (w3>=w1)&(w3<=w2)
          w2=w4;
          list(j,3)=0;
        end
      end
    end
    list_new=[list_new; w1 w2];
  end
end
list=list_new;
%
% ****************************************************************************
%
function list_intervals=mu_elim_ter(wmin,wmax,list_intervals)
%
% call:  list_intervals=mu_elim_ter(wmin,wmax,list_intervals)
%
% A subroutine of mu_elim.m
%
if nargin~=3 error('MU_ELIM_TER: 3 input arguments'); end
%
puls_add=[];
for i=1:size(list_intervals,1)
  if list_intervals(i,3)
    w1=list_intervals(i,1);
    w2=list_intervals(i,2);
    if (wmin>=w1)&(wmin<w2)
      list_intervals(i,3)=0;
      if wmax<w2
        if wmin>w1
          puls_add=[puls_add; w1 wmin 1; wmax w2 1];
        else
          puls_add=[puls_add; wmax w2 1];
        end
      elseif wmin>w1
        puls_add=[puls_add; w1 wmin 1];
      end
    elseif wmin<=w1
      if wmax>w1
        list_intervals(i,3)=0;
        if wmax<w2
          puls_add=[puls_add; wmax w2 1];
        end
      end
    end
  end
end
%
list_intervals=[list_intervals; puls_add];
%
% ****************************************************************************
%
function [wmin,wmax]=calc_interval_bis(sys,w0,D,G,beta,option)
%
% call:  [wmin,wmax]=calc_interval_bis(sys,w0,D,G,beta,option)
%
% A subroutine of mu_elim.m
%
% computation of the largest frequency interval [wmin,wmax] including w0
% in which D,G scaling matrices remain valid for a given value beta of the
% mu upper bound.
% 
% sys: system matrix of M(s) with ss format.
%
% option :
%   --> 0 (default):   D and G are "standard" scaling matrices associated to the LMI
%                      formulation of the mu upper bound  (D=D*>0)
%   --> 1          :   D and G are associated to the singular-value based
%                      formulation of the mu upper bound  (det(D)<>0)
%       
% Needed subroutines: exfreq1.m and dg2dgh.m
% 
if nargin==5
  option=0;
elseif nargin~=6
  error('CALC_INTERVAL_BIS : wrong number of input arguments');
end
%
[a,b,c,d]=ssdata(sys);
betar=sqrt(beta);
n=size(a,1);
r=size(D,1);
j=sqrt(-1);
%
% (D,G) --> (Dhat,Ghat)
%
if ~option
  [Dh,Gh]=dg2dgh(D,G,beta);
else
  Dh=D;
  Gh=G;
end
%
H=exfreq1(a,b,c,d,w0);
%
% Hcal computation
%
F=eye(r)+Gh^2;
F14=F^(-0.25);
F14a=mdiag(F14,eye(n));
Dha=mdiag(Dh/betar,eye(n));
Dhai=mdiag(inv(Dh)/betar,eye(n));
Gha=mdiag(j*Gh,zeros(n));
Ht=F14a*(Dha*H*Dhai-Gha)*F14a;     
Ht11=Ht(1:r,1:r);
Ht12=Ht(1:r,(r+1):(r+n));
Ht21=Ht((r+1):(r+n),1:r);
Ht22=Ht((r+1):(r+n),(r+1):(r+n));
X=[eye(r) Ht11;Ht11' eye(r)];
Hcal=mdiag(Ht22,Ht22')-([zeros(n,r) Ht21; Ht12' zeros(n,r)]/X)*mdiag(Ht12,Ht21');
%
warning off;
v=eig(Hcal);
warning on;
%
nn=n+n;
vpos=[]; vneg=[];
for ii=1:nn
  vr=real(v(ii)); 
  if vr~=0
    vi=imag(v(ii));
    if abs(vi/vr) < 1e-5   % a numerical tolerance, under which the eigenvalue is considered as real
      if vr > 0
        vpos=[vpos vr];
      else
        vneg=[vneg -vr];
      end
    end
  end
end
%
if isempty(vneg)
  wmin=-Inf;
else
  wmin=w0-min(1./vneg);
end 
wmin=max(wmin,0);
%
if isempty(vpos)
  wmax=Inf;
else
  wmax=w0+min(1./vpos);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Dh,Gh]=dg2dgh(D,G,beta)
%
% call:  [Dh,Gh]=dg2dgh(D,G,beta)
%
% Computation of scaling matrices Dhat, Ghat associated with the
% sigma max formulation of the mixed mu upper bound.
%
% The inputs are the "classical" scaling matrices D and G
% associated to the LMI formulation. beta is the mu upper bound.
%
n=size(D,1);
In=eye(n);
Dr=D^0.5;
Dri=In/Dr;
beta2=beta^2;
m=Dri*G*Dri'; 
m=((In+m'*m/beta2)^(0.25))*Dr;
[u,s,v]=svd(m);
p=v*s*v';
u=u*v';
Dh=p;
Gh=u'*Dri*G*Dri'*u/beta;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function H=exfreq1(a,b,c,d,w0)
%
% call:  H=exfreq1(a,b,c,d,w0)
%
% extract the frequency w which can then be treated
% as an uncertain real parameter.
%
% w=w0+dw --> d+c*(jwI-a)^(-1)*b = Fl(H, dw In)
%

warning off;

j=sqrt(-1);
r=sqrt(-j);

n=size(a,1);
I=eye(n);
x=((I+j*a/w0)\I)/w0;

H=[d-j*c*x*b r*c*x;-r*x*b j*a*x/w0-I/w0];

warning on;
