function [tab_mu,tab_pert]=mu_frequency_gridding(sys,blk,puls,method,alfa,cxi)
%
% call:  [tab_mu,tab_pert]=mu_frequency_gridding(sys,blk,puls,method,alfa,cxi)
%
% Robustness analysis of the standard interconnection structure M(s) - Delta.
%
% sys = system matrix (ss format) of M(s), a continuous-time plant.
%
% The structure of Delta is described by blk (musyn format).
% If blk has only 2 columns, a classical mu problem is considered.
% Otherwise the third column describes the potentially skew nature of the
% uncertainty.
%
% Delta and M(s) must be square.
%
% Computation of bounds of the (skew) s.s.v. at each point of a frequency gridding "puls".
%
% method = 1 --> computation of the mixed mu upper bound.
% tab_mu(iw) is the mu upper bound at frequency puls(iw). tab_pert=[].
%
% method = 2 --> computation of the mixed mu lower bound.
% tab_mu(iw) is the mu lower bound at frequency puls(iw).
% tab_pert(:,:,iw) is the structured model perturbation that destabilizes
% the closed loop at frequency puls(iw).
% In case of skew uncertainties (or if mu.m is not available) 
% an SMT version of the power-algorithm is used.
% Otherwise, the power-algorithm by musyn is used.
%
% method = 3 --> computation of a mu upper bound with Zadeh and
% Desoer's result. See the help of mu_zd.m inside this toolbox for
% further details. Delta must only contain non-repeated real
% uncertainties. The method is
% exponential-time, and thus limited to about 10 or 12 uncertain parameters.
%
% method = 4 --> computation of a mu lower bound with Dailey's method.
% See the help of mu_dailey.m inside this toolbox for further details.
% Delta must only contain non-repeated real uncertainties.
% The method is exponential-time, and thus limited
% to about 10 or 12 uncertain parameters.
%
% Robust stability is studied either inside the left half plane, or inside a
% truncated sector. alfa is the minimal degree of stability (alfa < 0 for
% a stable plant) and cxi is the minimal damping ratio. alfa=0 and cxi=0
% correspond to the left half plane. Only real uncertainties are allowed
% for robust stability inside a truncated sector, complex uncertainties
% are defined only on the imaginary axis.
%
% sys, blk, puls and method are mandatory. Other input arguments are
% optional ([] may also be used to specify them). Default values are
% alfa=0 and cxi=0.
%
% Skew Mu Toolbox - G. Ferreres and J.M. Biannic, Onera-DCSD Toulouse, France.
%
if nargin==4
  alfa=[];
  cxi=[];
elseif nargin==5
  cxi=[];
elseif nargin~=6
  error('MU_FREQUENCY_GRIDDING: wrong number of inputs arguments');
end
%
% Default values of input arguments
%
if isempty(alfa) alfa=0; end
if isempty(cxi) cxi=0; end
%
% Preliminary tests
%
idx=find(blk(:,1)<0);
if ~isempty(idx)
  qqq=blk(idx,2);
  if any(qqq~=0)
    error('MU_FREQUENCY_GRIDDING: full real blocks are not allowed');
  end
end
%
left_half_plane=(cxi==0)&(alfa==0);
%
if (~left_half_plane)&any(blk(:,1)>=0)
  error('MU_FREQUENCY_GRIDDING: complex uncertainties are undefined outside the imaginay axis');
end
%
if (method==3)|(method==4)
  if any(blk(:,1)~=-1)
     error('MU_FREQUENCY_GRIDDING: only non-repeated real uncertainties are allowed with this method');
  end 
end
%
if ~any(method==(1:4))
  error('MU_FREQUENCY_GRIDDING: Bad choice of method');
end
%
if size(blk,2)==2
  blk=[blk ones(size(blk,1),1)];
elseif size(blk,2)~=3
  error('MU_FREQUENCY_GRIDDING: blk must have 2 or 3 columns');
end
%
[a,b,c,d,Ts]=ssdata(sys);
if Ts~=0
  error('MU_FREQUENCY_GRIDDING: sys must be continuous-time');
end
%
e=eig(a);
[w,z]=damp(a);
z=min(z);
if (max(real(e))>=alfa)|(z<=cxi)
  error('MU_FREQUENCY_GRIDDING: nominal closed loop poles are not inside the left half plane or truncated sector');
end
%
% The case of rectangular model perturbations is not accounted for.
%
if size(d,1)~=size(d,2)
  error('MU_FREQUENCY_GRIDDING: M(s) must be square');
end
%
if ~square_delta(blk)
  error('MU_FREQUENCY_GRIDDING: Delta must be square');
end
%
nnn=sum(abs(blk(:,1)));
if size(d,1)~=nnn
  error('MU_FREQUENCY_GRIDDING: wrong respective sizes of M(s) and Delta');
end
%
if (method==2)&(any(blk(:,3)==0)|(exist('mu.m')~=2))
  [tab_mu,tab_pert]=mixed_mu_lb_freq(sys,blk,puls,1,alfa,cxi);
  return;
end
%
% Computation of the frequency response (possibly on the border of the truncated sector)
%
rep_M=calc_freq_resp(sys,puls,alfa,cxi);
%
fprintf('\n%i: ',length(puls));
tab_mu=[];
tab_pert=[];
for iw=1:length(puls)
  fprintf('%i.',iw);
  M=rep_M(:,:,iw);
  if method==1
    muu=mixed_mu_ub(M,blk);
    tab_mu(iw)=muu;
  elseif method==2
    [muu,prt]=mixed_mu_lb(M,blk,[1 0 0 0 0]);
    tab_mu(iw)=muu;
    tab_pert(:,:,iw)=prt;
  elseif method==3
    [bnds,prt]=mu_zd(M,blk);
    tab_mu(iw)=bnds;
    tab_pert(:,:,iw)=prt;
  elseif method==4
    [bnds,prt]=mu_dailey(M,blk);
    tab_mu(iw)=bnds;
    tab_pert(:,:,iw)=prt;
  end
end
fprintf('\n');
