function [tab_mu,puls_add,tab_pert]=mu_lb_with_freq(sys,blk,puls,method,alfa,cxi,epsilon,verbatim,target)
%
% call : [tab_mu,puls_add,tab_pert]=mu_lb_with_freq(sys,blk,puls,method,alfa,cxi,epsilon,verbatim,target)
%
% choose [] to use the default value, e.g.:
% [tab_mu,puls_add,tab_pert]=mu_lb_with_freq(sys,blk,puls,method,[],[],epsilon);
%
% the first four arguments are necessary, i.e. the minimal call is:
% [tab_mu,puls_add,tab_pert]=mu_lb_with_freq(sys,blk,puls,method);
%
% Computation of a mu lower bound for a transfer matrix M(s)
% (defined by system matrix sys, with ss format)
% and a model perturbation, whose structure is defined by "blk".
% A frequency gridding "puls" is used as an initial guess.
%
% THE AIM IS NOT TO COMPUTE A MU LOWER BOUND AS A FUNCTION OF FREQUENCY,
% BUT TO COMPUTE A LOWER BOUND OF THE MAXIMAL VALUE OF MU OVER THE FREQUENCY RANGE.
%
% NO SKEW UNCERTAINTY IS ALLOWED. If any, the 3d column of blk is not accounted for.
%
% ONLY THE CASE OF A REAL MODEL PERTURBATION IS TREATED.
%
% Instead of considering the left half plane, a truncated sector can be
% used: cxi is the minimal damping ratio, alfa the minimal
% degree of stability (the degree of stability of a matrix A is the
% maximal real part of the eigenvalues of A - alfa is thus negative if A
% is stable). alfa is required to be negative, cxi to be positive.
%
% verbatim = 0 --> no display
%          = 1 --> display of information at each frequency
%          = 2 --> minimal display
%
% tab_mu(i) corresponds to a lower bound of the real mu problem at puls_add(i),
% which is most generally different from puls(i).
% More precisely tab_mu(i) corresponds to the inverse of the norm 
% of the destabilizing model perturbation Delta =
% tab_pert(:,:,i), i.e. a pole of  A + B Delta C is on the imaginary axis
% at point j*puls_add(i) (with some numerical tolerance). NOTE THAT
% puls_add IS NOT SORTED, NEITHER IN ASCENDING NOR DESCENDING ORDER.
%
% If a mu lower bound is found, that is greater than the last input argument "target",
% the algorithm stops !
%
% *** For a basic use of the routine just choose method=4 or 2, epsilon=[].
% Otherwise, the use of these 2 arguments is explained in the following. ***
%
% A lower bound of a regularized mu problem is
% first computed at frequency puls(i), i.e. a small amount of complex
% uncertainties is added to the real model perturbation, to improve the
% convergence properties of the power algorithm which computes a mu lower
% bound (in the case of a purely real model perturbation, this algorithm
% often does not converge, so that no mu lower bound is computed).
%
% A destabilizing model perturbation Delta is provided with the mu
% lower bound. Nevertheless, this value is destabilizing for the regularized
% mu problem, not for the original one, since this model perturbation
% contains real and complex parts : Delta = diag(Delta_R,Delta_C). The
% real part Delta_R of the model perturbation is extracted and applied to
% the original real mu problem. Assume for the sake of simplicity that M(s) =
% (A,B,C,0), i.e. M(s) is a strictly proper transfer matrix (noting that
% the routine treats the general case of a simply proper transfer
% matrix). A pole of A + B Delta_R C is expected to be located near the point
% j*puls(i) of the imaginary axis (remember puls(i) is the frequency used
% for the regularised mu problem). Thus Delta_R is a good initial guess
% for a real destabilizing model perturbation. Four methods can be used now:
%
% method = 1 : the eigenvalue of A + B Delta_R C, which is the closest
% to the point j*puls(i) of the imaginary axis, is identified. If this
% eigenvalue is stable, the size of Delta = alfa * Delta_R is
% increased, until the corresponding eigenvalue of A + alfa B Delta_R C
% crosses the imaginary axis (a priori for alfa slightly greater than 1).
% Otherwise, the size of Delta is decreased, here again until the
% eigenvalue crosses the imaginary axis.
%
% method = 2 or 3 : the eigenvalue of A + B Delta_R C, which is the closest
% to the point j*puls(i) of the imaginary axis, is identified. When
% shifting this eigenvalue toward the imaginary axis, the norm of the model
% perturbation is minimized. This can be the infinity (method = 2)
% or two norm (method = 3).
%
% method = 4 : all eigenvalues of A + B Delta_R C are inspected. If one
% of them is already unstable, the size of Delta = alfa * Delta_R is
% decreased, until  A + alfa B Delta_R C becomes marginally stable (all
% eigenvalues in the left half plane or on the imaginary axis).
% Otherwise, the size of Delta = alfa * Delta_R is increased, until  A +
% alfa B Delta_R C becomes marginally stable.
%
% In all these methods, a regularized mu problem is computed at
% frequency puls(i), and the imaginary axis is crossed at puls_add(i).
% THIS FREQUENCY CAN BE FAR FROM puls(i).
%
% N.B. (1) : for a given frequency puls(i), the method may fail during the
% first or second step. If the mu problem is not regularized enough, the
% power algorithm may not converge. The migration toward the
% imaginary axis may also fail for numerical reasons. Remember that the aim is
% not to compute a mu lower bound as a function of frequency, but a lower
% bound of the maximal value of mu over the frequency range.
% Consequently, it is not especially troublesome if the method fails at a
% given frequency puls(i).
%
% N.B. (2) : even if method=3, tab_mu(i) is the inverse of the
% infinity norm of the destabilizing model perturbation Delta =
% tab_pert(:,:,i).
%
% epsilon is the amount of complex uncertainties, which are added to
% regularize the mu problem. epsilon = 5e-2 (the default value) seems to be enough
% in most situations.
%
% epsilon must be strictly positive.
%
% A full complex block is added to regularize the problem.
%
% When considering a truncated sector instead of the left half plane,
% the regularized mu problem is computed on the border of the
% sector, instead of the imaginary axis.
%
% When considering a robust stability problem inside the left half
% plane, the frequency omega corresponds to the point j*omega of the
% imaginary axis. In the case of a truncated sector, the frequency omega
% corresponds to the point xxx+j*omega of the border of the truncated
% sector, i.e. omega is the imaginary part of the point on the border.
%
% See for further details:
% G. Ferreres and J.M. Biannic
% "Reliable computation of the robustness margin for a flexible transport aircraft"
% Proceedings of the AIAA GNC 2000
% and Control Engineering Practice, pp. 1267-1278, vol. 9, 2001.
%
% Skew Mu Toolbox - G. Ferreres and J.M. Biannic, Onera-DCSD Toulouse, France.
%
if nargin==4
  alfa=[];
  cxi=[];
  epsilon=[];
  verbatim=[];
  target=[];
elseif nargin==5
  cxi=[];
  epsilon=[];
  verbatim=[];
  target=[];
elseif nargin==6
  epsilon=[];
  verbatim=[];
  target=[];
elseif nargin==7
  verbatim=[];
  target=[];
elseif nargin==8
  target=[];
elseif nargin~=9
  error('MU_LB_WITH_FREQ : wrong number of input arguments');
end
%
% Default values
%
if isempty(alfa) alfa=0; end
if isempty(cxi) cxi=0; end
if isempty(epsilon) epsilon=0.05; end
if isempty(verbatim) verbatim=1; end
if isempty(target) target=Inf; end
%
% No skew uncertainty is allowed. If any, the 3d column of blk is not accounted for.
%
blk=blk(:,1:2);
%
if epsilon<=0
  error('MU_LB_WITH_FREQ: epsilon must be strictly positive');
end
%
if all(method~=(1:4))
  error('MU_LB_WITH_FREQ : bad choice of input argument "method"');
end
%
if any(blk(:,1)>=0)|any(blk(:,2)~=0)
  error('MU_LB_WITH_FREQ: only real diagonal model perturbations are dealt with');
end
%
% size of the structured model perturbation
%
N=sum(abs(blk(:,1)));
%
if alfa>0
  fprintf('\n\n required degree of stability = %6.3f must be negative or zero\n\n',alfa);
  error('MU_LB_WITH_FREQ : ');
end
%
if cxi<0
  fprintf('\n\n required damping ratio = %6.3f must be positive or zero \n\n',cxi);
  error('MU_LB_WITH_FREQ : ');
end
%
sector=(alfa~=0)|(cxi~=0);
%
% Inspection of the nominal closed loop poles
%
[Asys,Bsys,Csys,Dsys]=ssdata(sys);
%
[ny,nu]=size(Dsys);
if ny~=nu
  error('M(s) is not square in MU_LB_WITH_FREQ');
end
if N~=ny
  error('Incompatible dimensions of the system and model perturbation in MU_LB_WITH_FREQ');
end
%
qqq=max(real(eig(Asys)));
[w,z]=damp(Asys);
z=min(z);
%
if (qqq>=alfa)|(z<=cxi)
  fprintf('\n\n nominal and required degrees of stability = %6.3f and %6.3f \n',qqq,alfa);
  fprintf('\n\n nominal and required damping ratios       = %6.3f and %6.3f \n\n',z,cxi);
  if sector
    error('MU_LB_WITH_FREQ : nominal closed loop poles do not belong to the specified sector of the complex plane !');
  else
    error('MU_LB_WITH_FREQ : unstable nominal closed loop poles');
  end
end
%
%  *** computation of the frequency response ***
%
g=calc_freq_resp(sys,puls,alfa,cxi);
%
if verbatim==1
  if method==1
    fprintf('\n\n regularized mu problem + shift of the target pole trough the imaginary axis\n');
    fprintf(' (expansion or decrease of the size of the initial guess for the real model perturbation)\n\n');
  elseif method==2
    fprintf('\n\n regularized mu problem + shift of the target pole trough the imaginary axis\n');
    fprintf(' (minimization of the infinity norm of the real model perturbation)\n\n');
  elseif method==3
    fprintf('\n\n regularized mu problem + shift of the target pole trough the imaginary axis\n');
    fprintf(' (minimization of the 2 norm of the real model perturbation)\n\n');
  elseif method==4
    fprintf('\n\n regularized mu problem + shift of any pole trough the imaginary axis\n');
    fprintf(' (expansion or decrease of the size of the initial guess for the real model perturbation)\n\n');
  end
end
%
nw=length(puls);
%
if verbatim==1
  fprintf('\n epsilon = %6.3e\n',epsilon);
  if epsilon
    fprintf('\n\n regularization by a full complex block\n\n');
  else
    fprintf('\n\n no regularization of the mu problem\n\n');
  end
elseif verbatim==2
  fprintf('\n %i: ',nw);
end
%
warning off;
tab_pert=NaN*ones(ny,nu,nw);
puls_add=[];
tab_mu=[];
for iw=1:nw
%
if verbatim==2 fprintf('%i.',iw); end
%
  M=g(:,:,iw);
%
  prt=NaN*ones(ny,nu);
  problem=0;
%
  if epsilon
    M_new=[M sqrt(epsilon)*M; sqrt(epsilon)*M epsilon*M];
    blk_new=[blk; size(M,1) size(M,1)];
  else
    M_new=M;
    blk_new=blk;
  end
  blk_new=[blk_new ones(size(blk_new,1),1)];
  [muu,prt_new]=mixed_mu_lb(M_new,blk_new,[1 0 0 0 0]);
%
  if norm(prt_new)>1e20 muu=0; end
%
  if muu==0
    omega=NaN;
    problem=1;
  else
    prt=prt_new(1:nu,1:nu);
    prt_first=prt;
    if method==4
      [muu,prt,omega]=mu_lb_1(sys,prt,sector,alfa,cxi);
    elseif (method==2)|(method==3)
      two_norm=(method==3);
      [muu,prt,omega]=mu_lb_2(two_norm,puls(iw),blk,sys,prt,sector,alfa,cxi);
    elseif method==1
      [muu,prt,omega]=mu_lb_3(puls(iw),sys,prt,sector,alfa,cxi);
    end
    if muu==0 problem=2; end
  end
  tab_mu=[tab_mu muu];
  puls_add=[puls_add omega];
  tab_pert(:,:,iw)=prt;
  prt_second=prt;
  if muu>target break; end
  if verbatim==1
    if sector==0
      fprintf('\n %3i: regularized mu problem at %6.3f rad/s --> mu = %6.3f - imaginary axis crossed at %6.3f rad/s',...
      iw,puls(iw),muu,omega);
    else
      fprintf('\n %3i: regularized mu problem at %6.3f rad/s --> mu = %6.3f - sector border crossed at %6.3f rad/s',...
      iw,puls(iw),muu,omega);
    end
    if problem==1
      fprintf('\n   (the power algorithm did not converge)');
    elseif problem==2
      fprintf('\n  (the pole(s) could not be shifted to the imaginary axis)'); 
    end
  end
%
end
if verbatim==1 fprintf('\n\n'); elseif verbatim==2 fprintf('\n'); end
warning on;
%
% ******************************************************************************************
%
function [muu,prt,omega]=mu_lb_1(sys,prt,sector,alfa,cxi)
%
% a subroutine of mu_lb_with_freq.m
%
[Asys,Bsys,Csys,Dsys]=ssdata(sys);
%
% preliminary test: is the initial model perturbation "prt" destabilizing ?
%
dalfa=1;
e=eig(Asys+dalfa*Bsys*prt*inv(eye(size(prt))-dalfa*Dsys*prt)*Csys);
%
if any(isnan(e))|any(isinf(e))
  muu=0;
  omega=NaN;
  prt=NaN*ones(size(prt));
  return;
end
%
if sector==0
  [qqq,i]=max(real(e));
  if qqq>=0
    unstable=1;
    omega=abs(imag(e(i)));
  else
    unstable=0;
  end
else
  [w,z]=damp(e);
  [alf_0,i1]=max(real(e));
  [cxi_0,i2]=min(z);
  if alf_0>=alfa
    unstable=1;
    omega=abs(imag(e(i1)));
  elseif cxi_0<=cxi
    unstable=1;
    omega=abs(imag(e(i2)));
  else
    unstable=0;
  end
end
%
% if the initial model perturbation is destabilizing, dalfa is decreased until
% a stabilizing model perturbation is found
%
% if the initial model perturbation is stabilizing, dalfa is increased until
% a destabilizing model perturbation is found
%
% After Nbr_iter_max iterations, the expansion factor is about expansion_dalfa^Nbr_iter_max
% the decreasing factor about decrease_dalfa^Nbr_iter_max
%
% 1.2^30 = 237.3763
% 0.8^30 =   0.0012
%
Nbr_iter_max=30;
expansion_dalfa=1.2;
decrease_dalfa=0.8;
%
Nbr_iter=0;
dalfa=1;
the_end=0;
while ~the_end
  if unstable==0
    dalfa=expansion_dalfa*dalfa;
  else
    dalfa=decrease_dalfa*dalfa;
  end
  e=eig(Asys+dalfa*Bsys*prt*inv(eye(size(prt))-dalfa*Dsys*prt)*Csys);
  if sector==0
    [qqq,i]=max(real(e));
    if qqq>=0
      unstable_bis=1;
      omega=abs(imag(e(i)));
    else
      unstable_bis=0;
    end
  else
    [w,z]=damp(e);
    [alf_0,i1]=max(real(e));
    [cxi_0,i2]=min(z);
    if alf_0>=alfa
      unstable_bis=1;
      omega=abs(imag(e(i1)));
    elseif cxi_0<=cxi
      unstable_bis=1;
      omega=abs(imag(e(i2)));
    else
      unstable_bis=0;
    end
  end
  the_end=(unstable_bis~=unstable);
  Nbr_iter=Nbr_iter+1;
  if Nbr_iter>Nbr_iter_max the_end=2; end
end
%
% The imaginary axis could not be crossed
%
if the_end==2
  muu=0;
  omega=NaN;
  prt=NaN*ones(size(prt));
else
%
% dichotomy search of the value of dalfa, for which
% the imaginary axis is crossed
%
  if unstable==0
    dalfa_min=1;
    dalfa_max=dalfa;
  else
    dalfa_min=dalfa;
    dalfa_max=1;
  end
%
  the_end=0;
  while ~the_end
    dalfa=(dalfa_min+dalfa_max)/2;
    e=eig(Asys+dalfa*Bsys*prt*inv(eye(size(prt))-dalfa*Dsys*prt)*Csys);
    if sector==0
      [qqq,i]=max(real(e));
      if qqq>=0
        omega=abs(imag(e(i)));
        dalfa_max=dalfa;
      else
        dalfa_min=dalfa;
      end
    else
      [w,z]=damp(e);
      [alf_0,i1]=max(real(e));
      [cxi_0,i2]=min(z);
      if alf_0>=alfa
        dalfa_max=dalfa;
        omega=abs(imag(e(i1)));
      elseif cxi_0<=cxi
        dalfa_max=dalfa;
        omega=abs(imag(e(i2)));
      else
        dalfa_min=dalfa;
      end
    end
    qqq=dalfa_max-dalfa_min;
    the_end=(qqq<(1e-10*dalfa_max)); % a very low value, so that we are very close to the imaginary axis 
  end                                % otherwise muu may not be a mu lower bound at frequency omega.
  prt=dalfa_max*prt;
  muu=1/norm(prt);
%
end
%
% ******************************************************************************************
%
function [muu,prt,omega]=mu_lb_2(twonorm,wp,blk,sys,prt,sector,alfa,cxi)
%
% a subroutine of mu_lb_with_freq.m
%
[Asys,Bsys,Csys,Dsys]=ssdata(sys);
%
if sector==0
  target_pt=sqrt(-1)*wp;
else
  theta=asin(cxi);
  q=-alfa*tan(pi/2-theta);
  if wp<=q
    target_pt=alfa+sqrt(-1)*wp;
  else
    target_pt=(-tan(theta)+sqrt(-1))*wp;
  end
end
%
Nbr_step=10;
%
problem=0;
the_end=0;
while ~the_end
%
  Abf=Asys+Bsys*prt*inv(eye(size(prt))-Dsys*prt)*Csys;
  [v,d]=eig(Abf);
  e=diag(d);
%
  if any(isnan(e))|any(isinf(e))|(norm(prt)>1e20)
    muu=0;
    omega=NaN;
    prt=NaN*ones(size(prt));
    return;
  end
%
  [qqq,i]=min(abs(e-target_pt));
  lambda=e(i);
  u=inv(v);
  v=v(:,i);
  u=u(i,:);
  w=prt*inv(eye(size(prt))-Dsys*prt)*Csys*v;
  t=u*Bsys*prt*inv(eye(size(prt))-Dsys*prt);
  A=[];
  nnn=size(blk,1);
  nnn_bis=sum(abs(blk(:,1)));
  icpt=1;
  for i=1:nnn
    qqq=abs(blk(i,1));
    idx=icpt:(icpt+qqq-1);
    icpt=icpt+qqq;
    Delta=zeros(1,nnn_bis);
    Delta(idx)=ones(size(idx));
    Delta=diag(Delta);
    qqq=(u*Bsys+t*Dsys)*Delta*(Csys*v+Dsys*w);
    A(1,i)=real(qqq);
  end
  qqq=(target_pt-lambda);
  b=real(qqq)/Nbr_step;
  Nbr_step=Nbr_step-1;
  the_end=(Nbr_step==1);
  if twonorm
    AA=[2*eye(nnn) A'; A zeros(1,1)];
    bb=[zeros(nnn,1); b];
    if rcond(AA)<=eps
      qqq=NaN;
    else
      qqq=AA\bb;
    end
  else
%    if 0
% there seems to be some trouble with linprog.m ....
%      qqq=mu_lb_2_1(A,b,blk,nnn,prt);
%    else
      qqq=mu_lb_2_2(A,b,blk,nnn,prt);
%    end
  end
  if any(any(isinf(qqq)))|any(any(isnan(qqq)))
    problem=1;
    break;
  end
  Delta=[];
  Delta_bis=qqq(1:nnn);
  icpt=1;
  for i=1:nnn
    qqq=abs(blk(i,1));
    idx=icpt:(icpt+qqq-1);
    icpt=icpt+qqq;
    Delta(idx)=Delta_bis(i)*ones(size(idx));
  end
  prt=prt+diag(Delta);
end
%
if problem
  muu=0;
  omega=NaN;
  prt=NaN*ones(size(prt));
else
  [muu,prt,omega]=mu_lb_3(wp,sys,prt,sector,alfa,cxi);
end
%
% ******************************************************************************************
%
function  qqq=mu_lb_2_1(A,b,blk,nnn,prt)
%
% a subroutine of mu_lb_2.m
%
delta_ini=[];
icpt=1;
for i=1:nnn
  delta_ini(i)=prt(icpt,icpt);
  qqq=abs(blk(i,1));
  icpt=icpt+qqq;
end
delta_ini=delta_ini(:);
%
% -alfa < delta + delta_i < alfa
%
% is equivalent to :
%
% -delta-alfa <  delta_i
%  delta-alfa < -delta_i
%
% with the equality constraint A * delta = b
%
AA=[-eye(nnn) -ones(nnn,1); eye(nnn) -ones(nnn,1)];
bb=[delta_ini; -delta_ini];
f=zeros(nnn+1,1);
f(nnn+1)=1;
%
A=[A zeros(size(A,1),1)];
%
options=optimset('Display','none','LargeScale','off');
qqq=linprog(f,AA,bb,A,b,[],[],[],options);
%
% ******************************************************************************************
%
function  qqq=mu_lb_2_2(A,b,blk,nnn,prt)
%
% a subroutine of mu_lb_2.m
%

%
% A*delta = b --> delta = N*x+delta_0 with delta_0=A\b
%
Nker=null(A);
delta_0=A\b;
%
delta_i=[];
icpt=1;
for i=1:nnn
  delta_i(i)=prt(icpt,icpt);
  qqq=abs(blk(i,1));
  icpt=icpt+qqq;
end
delta_i=delta_i(:);
%
% the infinity norm of delta+delta_i is to minimized,
% the issue is thus to  minimize alfa under the constraints:
%
% -alfa < N*x+delta_0+delta_i < alfa
%
% equivalently :
%
%  N*x-alfa < -delta_0-delta_i 
%
% -N*x-alfa < delta_0+delta_i
%
AA=[];
bb=[];
for i=1:nnn
  AA=[AA;  Nker(i,:) -1; -Nker(i,:) -1];
  bb=[bb; -delta_0(i)-delta_i(i); delta_0(i)+delta_i(i)];
end
%
x0=zeros(size(Nker,2),1);
alfa=1.01*norm(delta_0+delta_i,Inf);
%
x0=[x0; alfa];
%
nnn_bis=size(Nker,2);
f=zeros(1,nnn_bis+1);
f(nnn_bis+1)=1;
%
x=mu_lb_2_3(f,AA,bb,x0);
alfa=x(nnn_bis+1);
x=x(1:nnn_bis);
%
qqq=[Nker*x+delta_0; alfa];
%
% ******************************************************************************************
%
function  x=mu_lb_2_3(f,A,b,x0)
%
% a subroutine of mu_lb_2_2.m
%
x0=x0(:);
b=b(:);
f=f(:)';
%
[N,mK]=size(A);
%
constraint=A*x0-b;
%
x=x0;
H=eye(mK); Hinv=inv(H);
I=zeros(1,mK);
%
global COUNTER;
COUNTER=0;
COUNTER_max=30;
%
icpt=0;
the_end=0;
while ~the_end
%
  if any(I==0)
    list=find(I==0);
    q1=f*Hinv(:,list);
    [q2,i]=max(abs(q1));
    d=list(i);
    if q2==0
      OK=0;
    else
      OK=1;
      if q1(i)>=0 s=Hinv(:,d); else s=-Hinv(:,d); end
    end
  else
    OK=0;
  end
%
  if ~OK
    list=find(I~=0);
    q1=f*Hinv(:,list);
    [q2,i]=max(q1);
    d=list(i);
    if q2<=0
      the_end=1;
      break;
    else
      s=Hinv(:,d); 
    end
  end
%
  w=A*s;
  if min(w)>0 the_end=2; break; end
  ww=w;
  for i=I
    if i>0 ww(i)=1; end
  end
  list=find(ww<0);
  [sig,i]=min(constraint(list)./w(list));
  l=list(i);
  I(d)=l;
  H(d,:)=A(l,:);
  Hinv=inv(H);
  x=x-sig*s;
  constraint=constraint-sig*w;
%
  qqq=cond(H);
  if qqq>1e16
    COUNTER=COUNTER+1;
    if COUNTER>COUNTER_max the_end=3; end
  end
%  
end
%
% ******************************************************************************************
%
function  [muu,prt,omega]=mu_lb_3(wp,sys,prt,sector,alfa,cxi)
%
% a subroutine of mu_lb_with_freq.m
%
if sector==0
  target_pt=sqrt(-1)*wp;
else
  theta=asin(cxi);
  q=-alfa*tan(pi/2-theta);
  om=[];
  if wp<=q
    target_pt=alfa+sqrt(-1)*wp;
   else
    target_pt=(-tan(theta)+sqrt(-1))*wp;
  end
end
%
[Asys,Bsys,Csys,Dsys]=ssdata(sys);
%
e=eig(Asys+Bsys*prt*inv(eye(size(prt))-Dsys*prt)*Csys);
if any(isnan(e))|any(isinf(e))
  muu=0;
  omega=NaN;
  prt=NaN*ones(size(prt));
  return;
end
%
[qqq,i]=min(abs(e-target_pt));
lambda=e(i);
if sector==0
  if real(lambda)>=0
    unstable=1;
    omega=abs(imag(lambda));
  else
    unstable=0;
  end
else
  [w,z]=damp(lambda);
  if (real(lambda)>=alfa)|(z<=cxi)
    unstable=1;
    omega=abs(imag(lambda));
  else
    unstable=0;
  end
end
%
% if the initial model perturbation is destabilizing, dalfa is decreased until
% a stabilizing model perturbation is found
%
% if the initial model perturbation is stabilizing, dalfa is increased until
% a destabilizing model perturbation is found
%
% After Nbr_iter_max iterations, the expansion factor is about expansion_dalfa^Nbr_iter_max
% the decreasing factor about decrease_dalfa^Nbr_iter_max
%
% 1.1^60 = 304.4816
% 0.9^60 =   0.0018
%
Nbr_iter_max=60;
expansion_dalfa=1.1;
decrease_dalfa=0.9;
%
Nbr_iter=0;
dalfa=1;
the_end=0;
while ~the_end
  if unstable==0
    dalfa=expansion_dalfa*dalfa;
  else
    dalfa=decrease_dalfa*dalfa;
  end
  e=eig(Asys+dalfa*Bsys*prt*inv(eye(size(prt))-dalfa*Dsys*prt)*Csys);
  [qqq,i]=min(abs(e-target_pt));
  lambda=e(i);
  if sector==0
    if real(lambda)>=0
      unstable_bis=1;
      omega=abs(imag(lambda));
    else
      unstable_bis=0;
    end
  else
    [w,z]=damp(lambda);
    if (real(lambda)>=alfa)|(z<=cxi)
      unstable_bis=1;
      omega=abs(imag(lambda));
    else
      unstable_bis=0;
    end
  end
  the_end=(unstable_bis~=unstable);
  Nbr_iter=Nbr_iter+1;
  if Nbr_iter>Nbr_iter_max the_end=2; end
end
%
if the_end==2
  muu=0;
  omega=NaN;
  prt=NaN*ones(size(prt));
else
  if unstable==0
    dalfa_min=1;
    dalfa_max=dalfa;
  else
    dalfa_min=dalfa;
    dalfa_max=1;
  end
  the_end=0;
  while ~the_end
    dalfa=(dalfa_min+dalfa_max)/2;
    e=eig(Asys+dalfa*Bsys*prt*inv(eye(size(prt))-dalfa*Dsys*prt)*Csys);
    [qqq,i]=min(abs(e-target_pt));
    lambda=e(i);
    if sector==0
      if real(lambda)>=0
        dalfa_max=dalfa;
        omega=abs(imag(lambda));
      else
        dalfa_min=dalfa;
      end
    else
      [w,z]=damp(lambda);
      if (real(lambda)>=alfa)|(z<=cxi)
        dalfa_max=dalfa;
        omega=abs(imag(lambda));
      else
        dalfa_min=dalfa;
      end
    end
    qqq=dalfa_max-dalfa_min;
    the_end=(qqq<(1e-10*dalfa_max)); % a very low value, so that we are very close to the imaginary axis
  end                                % otherwise muu may not be a mu lower bound at frequency omega.
  prt=dalfa_max*prt;
  muu=1/norm(prt);
end
