function [tab_beta_visu,tab_puls_visu]=mu_max_1(a1,a2,a3,a4,a5,a6)
%
% call:  [tab_beta_visu,tab_puls_visu]=mu_max_1(sys,blk,puls)
%        [tab_beta_visu,tab_puls_visu]=mu_max_1(sys,blk,puls,num_figure)
%        [tab_beta_visu,tab_puls_visu]=mu_max_1(sys,blk,puls,alfa,cxi)
%        [tab_beta_visu,tab_puls_visu]=mu_max_1(sys,blk,puls,alfa,cxi,num_figure)
%
% Robust stability of the standard interconnection structure M(s) - Delta.
%
% sys = system matrix (ss format) of M(s), a continuous-time plant.
%
% The structure of Delta is described by blk (musyn format).
% The third column describes the potentially skew nature of
% the uncertainty: 
%
% blk(i,3) = 0 --> norm of the uncertainty less than 1.
%          = 1 --> norm of the uncertainty less than k, where k is maximized.
%
% Delta and M(s) must be square.
%
% puls = initial frequency gridding.
%
% Computation of a guaranteed mu upper bound on a union of small frequency intervals.
% The computation is performed either on the imaginary axis or on the borderline 
% of a truncated sector. In that case, only real uncertainties are allowed.
% alfa is the minimal degree of stability (alfa < 0 for a stable plant) 
% and cxi is the minimal damping ratio. (alfa=0 and cxi=0 correspond to the LHP)
% 
% Let [wmin, wmax] a small interval. D,G scaling matrices are computed, which are valid
% at wmin and wmax. It is then checked whether these scaling matrices are valid
% inside the associated interval. If yes, a guaranteed mu upper bound has been computed
% on the interval.
%
% tab_beta_visu(i) is a guaranteed mu upper bound over the
% frequency interval [tab_puls_visu(i,1), tab_puls_visu(i,2)]. Results can be visualized
% with plot_bar.m
%
% It may be useless to use a too fine initial gridding puls, this may even be
% time consuming. D,G scaling matrices are first computed at puls(i) and puls(i+1).
% If they are not valid inside the associated interval this one is split into
% [puls(i), (puls(i)+puls(i+1))/2] and [(puls(i)+puls(i+1))/2, puls(i+1)].
%
% Nevertheless, assume that a guaranteed mu upper bound was computed on a frequency interval.
% It may happen that when splitting this frequency interval into smaller ones,
% the maximal guaranteed mu upper bound over these smaller intervals is less than the initial
% guaranteed mu upper bound.
%
% num_figure = 0 --> no display
%              I --> display of intermediate results on figure number I.
%
% Defaut value for num_figure is 0.
%
% See for further details:
% G. Ferreres and J.M. Biannic, Proc. of the ACC 1998,
% "A mu analysis technique without frequency gridding",
% pp. 2294-2298, vol. 4.
%
% Skew Mu Toolbox - G. Ferreres and J.M. Biannic, Onera-DCSD Toulouse, France.
%

sys=a1; blk=a2; puls=a3;
if nargin == 4,
  num_figure=a4; alfa=0; cxi=0;
elseif nargin == 5,
   num_figure=0;
   alfa=a4;
   cxi=a5;
elseif nargin==6, 
   alfa=a4; cxi=a5; num_figure=a6;
else  
   error('MU_MAX_1: wrong number of input arguments');
end;

alfa=abs(alfa); 

%
if num_figure fid=2; else fid=0; end
%
% Nbr_freq_LMI = number of frequencies, which are used to compute D,G scaling matrices.
% Let [wmin,wmax] the frequency interval.
% If Nbr_freq_LMI=2 D,G scaling matrices are computed at wmin and wmax.
% If Nbr_freq_LMI=3 they are computed at wmin, (wmin+wmax)/2 and wmax.
%
Nbr_freq_LMI=2;
%
% Preliminary tests
%
idx=find(blk(:,1)<0);
if ~isempty(idx)
  qqq=blk(idx,2);
  if any(qqq~=0)
    error('MU_MAX_1: full real blocks are not allowed');
  end
end
%
if size(blk,2)~=3
  error('MU_MAX_1: blk must have 3 columns');
end
%
if all(blk(:,3)==0)
  error('MU_MAX_1: problem with the 3rd column of blk');
end
%
[a,b,c,d,Ts]=ssdata(sys);
if Ts~=0
  error('MU_MAX_1: sys must be continuous-time');
end
%
left_half_plane=(cxi==0)&(alfa==0);
if (~left_half_plane)&any(blk(:,1)>=0)
  error('MU_MAX_1: complex uncertainties are undefined outside the imaginay axis');
end
%
[wa,za,e]=damp(a);
alfa_min=-max(real(e));
cxi_min=min(za);
if (cxi > cxi_min)|(alfa > alfa_min)
  error('MU_MAX_1: nominal  poles are not inside LHP or specified truncated sector');
end
%
% The case of rectangular model perturbations is not accounted for.
%
if size(d,1)~=size(d,2)
  error('MU_MAX_1: M(s) must be square');
end
%
if ~square_delta(blk)
  error('MU_MAX_1: Delta must be square');
end
%
nnn=sum(abs(blk(:,1)));
if size(d,1)~=nnn
  error('MU_MAX_1: wrong respective sizes of M(s) and Delta');
end
%
[sys,blk]=skew_sort(sys,blk,1);
%
puls=sort(puls); 
npuls=length(puls);
w_sec=0;
if alfa*cxi>0,
    theta=asin(cxi);
    w_sec=alfa/tan(theta);
    ii=1; puls1=[];
    while puls(ii)<w_sec
      puls1=[puls1 puls(ii)];
      ii=ii+1;
    end;
    puls1=[puls1 w_sec];
    puls2=puls(ii:npuls);
    if puls2(1)>w_sec,
      puls2=[w_sec puls2];
    end; 
else,
  puls1=puls;
end;  

tab_puls_visu=[];
tab_beta_visu=[];


%%%%%%%%%%%%%%%%%%%%%%%

if num_figure
  figure(num_figure);
  clf;
  first_time=1;
end

%
disp(' ');
if w_sec==0,
  nsec=1;
  disp('Computations will be performed on one segment'); disp(' ');
else
  nsec=2;
  disp('Computations will be performed on two segments'); disp(' ');
end;

disp(' ');
if nsec==1,
 if cxi==0,
  if alfa==0,
    disp(['SEGMENT 1 : standard stability constraint']);
    disp('-----------------------------------------');  
  else,
    disp(['SEGMENT 1 : alpha-stability constraint']);
    disp('---------------------------------------');  
  end;
 else
   disp(['SEGMENT 1 : damping constraint']);
   disp('-------------------------------');  
 end;
end;
 
for isec=1:nsec,
if nsec==2,
 if isec==1,
  disp(' ');
  disp(['SEGMENT 1 : alpha-stability constraint']);
  disp('---------------------------------------');
 else
  disp(' ');
  disp(['SEGMENT 2 : damping constraint']);
  disp('-------------------------------');
 end;
end;
eval(['puls=puls',num2str(isec),';']);

tab_puls=[];
for i=1:(length(puls)-1)
  tab_puls=[tab_puls; puls(i) puls(i+1)];
end


the_end=0;
while ~the_end
%
  nw=size(tab_puls,1);
%
% Computation of a (skew) mu upper bound, which is simultaneously valid
% at Nbr_freq_LMI frequencies
%
  tab_beta=[];
  tab_D=[];
  tab_G=[];
  tab_H=[];
  fprintf(fid,'\n %i: ',nw);
  for i=1:nw
    fprintf(fid,'%i.',i);
    freq=linspace(tab_puls(i,1),tab_puls(i,2),Nbr_freq_LMI);
    rep_M=calc_freq_resp(sys,freq,alfa,cxi);
    [beta,H,D,G]=skew_mu_ub_bis(rep_M,blk,1e-2,1,1e-3);
    beta=max(beta,1e-3);          % beta must not be zero     
    tab_beta=[tab_beta beta];
    tab_D(:,:,i)=D;
    tab_G(:,:,i)=G;
    tab_H(:,:,i)=H;
  end
  fprintf(fid,'\n');
%
  tab_puls_supp=[];
  tab_puls_new=[];
  tab_beta_new=[];
%
% Are D,G scaling matrices valid on the whole interval ? If not
% this interval is split into 2 smaller ones.
%
  for i=1:nw
%
    D=tab_D(:,:,i);
    G=tab_G(:,:,i);
    H=tab_H(:,:,i);
    beta=tab_beta(i);
    if isinf(beta)
      OK=1;
    else
      [a,b,c,d]=ssdata(sys);
      sys_new=ss(a,b,H*c,H*d);
      w1=tab_puls(i,1);
      w2=tab_puls(i,2);
      w=(w1+w2)/2;
      Mw=calc_freq_resp(sys_new,w,alfa,cxi);
      testDGw=testdg(Mw,D,G,beta);   
      if  testDGw
        [wmin,wmax]=calc_interval(sys_new,w,D,G,beta,alfa,cxi);
        if (wmin<=w1)&(wmax>=w2)
          OK=1;
        else
          OK=0;
        end
      else
        OK=0;
      end
    end
 %   
    if ~OK
      tab_puls_supp=[tab_puls_supp; tab_puls(i,1) w; w tab_puls(i,2)];
      fprintf(fid,'\n %i: scaling matrices validity problem for w between  %8.5f and %8.5f with mu = %8.5f\n',...
      i,tab_puls(i,1),tab_puls(i,2),beta);
    else
      tab_puls_new=[tab_puls_new; tab_puls(i,:)];
      tab_beta_new=[tab_beta_new tab_beta(i)];
    end
%
  end
%
  tab_puls_visu=[tab_puls_visu; tab_puls_new];
  tab_beta_visu=[tab_beta_visu  tab_beta_new];
  
  
%
% Visualisation of the result
%
  if num_figure
    if first_time first_time=0; else hold on; end
    plot_bar(tab_puls_visu,tab_beta_visu,'lin');
    hold off;
    figure(num_figure);
  end
%
  if ~isempty(tab_puls_supp)
    tab_puls=tab_puls_supp;
  else
    the_end=1;
  end
%
end  % the_end

end; % isec


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [wmin,wmax,tabv]=calc_interval(sys,w0,D,G,beta,alfa,cxi,option,option2)
%
% call:  [wmin,wmax,tabv]=calc_interval(sys,w0,D,G,beta{,alfa,cxi,option1,option2})
%
% computation of the largest frequency interval [wmin,wmax] including w0
% in which D,G scaling matrices remain valid for a given value beta of the
% mu upper bound.
%
% 
% sys: system matrix of M(s) with ss format.
%
% option1 :
%   --> 0 (default) :   D and G are "standard" scaling matrices associated to the LMI
%                       formulation of the mu upper bound  (D=D*>0)
%   --> 1           :   D and G are associated to the singular-value based
%                       formulation of the mu upper bound  (det(D)<>0)
% option2 :
%   --> 0 (default) :   only main segment is computed
%   --> 1           :   if any, secondary frequency segments are provided
%
if nargin <9,
    option2=0;
end;
if nargin<5,
  error('CALC_INTERVAL : wrong number of input arguments');
end;
if nargin==7,
 option=0;
end;
if nargin==6,
 option=alfa;
 alfa=0; cxi=0;
end;
if nargin==5,
 option=0; alfa=0; cxi=0;
end;
alfa=abs(alfa);
w_sec=0;
if cxi>0,
    theta=asin(cxi);
    tanth=tan(theta);
    w_sec=alfa/tanth;
end

%
[a,b,c,d]=ssdata(sys);
betar=sqrt(beta);
n=size(a,1);
r=size(D,1);
j=sqrt(-1);

if (cxi>0) & (w0>w_sec),
  z=1+j*tanth; zsq=sqrt(z);
  a=a/z;
  b=b/zsq; c=c/zsq;
else,
  a=a+alfa*eye(n);
end;   

%
% (D,G) --> (Dhat,Ghat)
%
if ~option
  [Dh,Gh]=dg2dgh(D,G,beta);
else
  Dh=D;
  Gh=G;
end
%
H=exfreq1(a,b,c,d,w0);
%
% Hcal computation
%
F=eye(r)+Gh^2;
F14=F^(-0.25);
%F14=sqrtm(sqrtm(inv(F)));
F14a=mdiag(F14,eye(n));
Dha=mdiag(Dh/betar,eye(n));
Dhai=mdiag(inv(Dh)/betar,eye(n));
Gha=mdiag(j*Gh,zeros(n));
Ht=F14a*(Dha*H*Dhai-Gha)*F14a;     
Ht11=Ht(1:r,1:r);
Ht12=Ht(1:r,(r+1):(r+n));
Ht21=Ht((r+1):(r+n),1:r);
Ht22=Ht((r+1):(r+n),(r+1):(r+n));
X=[eye(r) Ht11;Ht11' eye(r)];
Hcal=mdiag(Ht22,Ht22')-([zeros(n,r) Ht21; Ht12' zeros(n,r)]/X)*mdiag(Ht12,Ht21');
%
v=eig(Hcal);
%
nn=n+n;
vpos=[]; vneg=[];
for ii=1:nn
  vr=real(v(ii)); 
  if vr~=0
    vi=imag(v(ii));
    if abs(vi/vr) < 1e-5   % a numerical tolerance, under which the eigenvalue is considered as real
      if vr > 0
        vpos=[vpos vr];
      else
        vneg=[vneg -vr];
      end
    end
  end
end
%
if isempty(vneg)
  wmin=-Inf;
  dwneg=[];
else
  dwneg=sort(1./vneg);
  wmin=w0-dwneg(1);
end 
wmin=max(wmin,0);
%
if isempty(vpos)
  wmax=Inf;
  dwpos=[];
else
  dwpos=sort(1./vpos);
  wmax=w0+dwpos(1);
end

if (alfa*cxi) > 0
  if w0<w_sec,
     wmax=min(wmax,w_sec);
  else,
     wmin=max(wmin,w_sec);
  end;
end;
tabv=[wmin wmax];

if option2,
    
    pulsp=w0+dwpos;
    pulsn=w0-dwneg; 

    np=length(pulsp);
    testp=[];
    for ii=1:np-1,
        wi=(pulsp(ii)+pulsp(ii+1))/2;
        Mi=calc_freq_resp(sys,wi,alfa,cxi);
        Zi=F14*((Dh*Mi/Dh)/beta - j*Gh)*F14; 
        if norm(Zi)<1,
            tabv=[tabv;[1.001*pulsp(ii) 0.999*pulsp(ii+1)]];
        end;
    end;
    nn=length(pulsn);
    testn=[];
    for ii=1:nn-1,
        wi=(pulsn(ii)+pulsn(ii+1))/2;
        Mi=calc_freq_resp(sys,wi,alfa,cxi);
        Zi=F14*((Dh*Mi/Dh)/beta - j*Gh)*F14; 
        if norm(Zi)<1,
            tabv=[tabv;[ 1.001*pulsn(ii+1) 0.999*pulsn(ii)]];
        end;
    end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Dh,Gh]=dg2dgh(D,G,beta)
%
% call:  [Dh,Gh]=dg2dgh(D,G,beta)
%
% Computation of scaling matrices Dhat, Ghat associated with the
% sigma max formulation of the mixed mu upper bound.
%
% The inputs are the "classical" scaling matrices D and G
% associated to the LMI formulation. beta is the mu upper bound.
%
n=size(D,1);
In=eye(n);
Dr=D^0.5;
Dri=In/Dr;
beta2=beta^2;
m=Dri*G*Dri'; 
m=((In+m'*m/beta2)^(0.25))*Dr;
[u,s,v]=svd(m);
p=v*s*v';
u=u*v';
Dh=p;
Gh=u'*Dri*G*Dri'*u/beta;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [test,vs] = testdg(a1,a2,a3,a4,a5,a6)
%
% calls:  test = testdg(sys,w,D,G,beta,option)
%         test = testdg(sys,w,D,G,beta)
%         test = testdg(M,D,G,beta,option)
%         test = testdg(M,D,G,beta)
%
% This function tests the validity of scaling matrices associated
%   - to the LMI formulation of the mixed-mu upper-bound : ... option==0
%   - to the singular-value test : ........................... option==1 
%

if isa(a1,'lti'),
  sys=a1;
  w=a2;
  M=freqresp(sys,w);
  if nargin < 6,
    option=0;
  else
    option=a6;
  end;
  D=a3; G=a4; beta=a5;
else,
  M=a1;
  D=a2; G=a3; beta=a4;
  if nargin<5,
    option=0;
  else
    option=a5;
  end;
end;

if ~option,  % LMI case

aux=sqrt(-1)*G*M;
H=M'*D*M+aux+aux'-(beta^2)*D;
v=eig(H);
vs=max(real(v));
if vs<=0
  test=1;
else
  test=0;
end


else   % singular value case

n=size(M,1);
F=(eye(n)+G*G)^(-0.25);
Z=F*((D*M/D)/beta - j*G)*F; 
vs=norm(Z);
if vs < 1,  
  test=1;
else
  test=0;
end;

end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function H=exfreq1(a,b,c,d,w0)
%
% call:  H=exfreq1(a,b,c,d,w0)
%
% extract the frequency w which can then be treated
% as an uncertain real parameter.
%
% w=w0+dw --> d+c*(jwI-a)^(-1)*b = Fl(H, dw In)
%

warning off;

j=sqrt(-1);
r=sqrt(-j);

n=size(a,1);
I=eye(n);
x=((I+j*a/w0)\I)/w0;

H=[d-j*c*x*b r*c*x;-r*x*b j*a*x/w0-I/w0];

warning on;
