function  [tab_beta_visu,tab_puls_visu]=mu_max_2(sys,blk,puls)
%
% call:   [tab_beta_visu,tab_puls_visu]=mu_max_2(sys,blk,puls)
%
% Robust stability of the standard interconnection structure M(s) - Delta.
%
% sys = system matrix (ss format) of M(s), a continuous-time plant.
%
% The structure of Delta is described by blk (musyn format).
% The third column describes the potentially skew nature of
% the uncertainty: 
%
% blk(i,3) = 0 --> norm of the uncertainty less than 1.
%          = 1 --> norm of the uncertainty less than k, where k is maximized.
%
% Delta and M(s) must be square.
%
% puls = frequency gridding.
%
% Computation of a guaranteed mu upper bound on a union of frequency intervals.
% tab_beta_visu(i) is a guaranteed mu upper bound over the
% frequency interval [puls(i), puls(i+1)]. This frequency interval is renamed as
% [tab_puls_visu(i,1), tab_puls_visu(i,2)].
%
% The frequency gridding puls is not refined inside the routine, unlike in mu_max_1.m.
%
% Nevertheless, assume that a guaranteed mu upper bound was computed on a frequency interval.
% It may happen that when splitting this frequency interval into smaller ones,
% the maximal guaranteed mu upper bound over these smaller intervals is less than the initial
% guaranteed mu upper bound.
%
% Results can be visualized with plot_bar.m
%
% See for further details:
% G. Ferreres and V. Fromion,
% "Computation of the robustness margin with the skewed mu tool",
% Systems and Control Letters,
% Vol. 32, n. 4, pp. 193-202, December 1997.
%
% Needed subroutines: exfreq1.m and skew_mu_ub.m
%
% Skew Mu Toolbox - G. Ferreres and J.M. Biannic, Onera-DCSD Toulouse, France.
%
if nargin~=3 error('MU_MAX_2; 3 input arguments'); end
%
% Preliminary tests
%
idx=find(blk(:,1)<0);
if ~isempty(idx)
  qqq=blk(idx,2);
  if any(qqq~=0)
    error('MU_MAX_2: full real blocks are not allowed');
  end
end
%
if size(blk,2)~=3
  error('MU_MAX_2: blk must have 3 columns');
end
%
if all(blk(:,3)==0)
  error('MU_MAX_2: problem with the 3th column of blk');
end
%
[a,b,c,d,Ts]=ssdata(sys);
if Ts~=0
  error('MU_MAX_2: sys must be continuous-time');
end
%
e=eig(a);
if max(real(e))>=0
  error('MU_MAX_2: nominal closed loop poles are not inside the left half plane');
end
%
% The case of rectangular model perturbations is not accounted for.
%
if size(d,1)~=size(d,2)
  error('MU_MAX_2: M(s) must be square');
end
%
if ~square_delta(blk)
  error('MU_MAX_2: Delta must be square');
end
%
nnn=sum(abs(blk(:,1)));
if size(d,1)~=nnn
  error('MU_MAX_2: wrong respective sizes of M(s) and Delta');
end
%
puls=sort(puls);
[nx,nu]=size(b);
blk_aug=[blk; -nx 0 0];
%
tab_beta_visu=[];
tab_puls_visu=[];
%
fprintf('\n%i: ',length(puls)-1);
for iw=1:(length(puls)-1)
%
  fprintf('%i.',iw);
%
  w1=puls(iw);
  w2=puls(iw+1);
  tab_puls_visu=[tab_puls_visu; w1 w2];
%
  w0=(w1+w2)/2;
  delta_w=(w2-w1)/2;
  H=exfreq1(a,b,c,d,w0);
  I=eye(nx+nu);
  idx=(nu+1):(nu+nx);
  I(idx,idx)=sqrt(delta_w)*eye(nx);
  H=I*H*I;
%
  beta=skew_mu_ub(H,blk_aug,1e-2,1,1e-3);
  tab_beta_visu=[tab_beta_visu beta];
%
end
fprintf('\n');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function H=exfreq1(a,b,c,d,w0)
%
% call:  H=exfreq1(a,b,c,d,w0)
%
% extract the frequency w which can then be treated
% as an uncertain real parameter.
%
% w=w0+dw --> d+c*(jwI-a)^(-1)*b = Fl(H, dw In)
%
j=sqrt(-1);
r=sqrt(-j);

n=size(a,1);
I=eye(n);
x=((I+j*a/w0)\I)/w0;

H=[d-j*c*x*b r*c*x;-r*x*b j*a*x/w0-I/w0];
