function [muu,D,G]=mu_ub(a1,a2,a3,a4,a5,a6)
%
% calls: [muub,D,G]=mu_ub(M,blk,tol,trace,target)
%        [muub,D,G]=mu_ub(sys_M,puls,blk,tol,trace,target)
%
% LMI-based computation of a mixed mu upper bound,
% which is simultaneously valid for several values of matrix M.
%
% These values may correspond to  M(:,:,i) for i between 1 and N (1st call),
% or to the frequency response of system matrix sys_M (ss format)
% on the frequency gridding puls (2d call).
%
% The structure of the model perturbation Delta is described by blk.
% No skew uncertainty is accounted for, i.e. if any the third column of blk
% is not accounted for.
%
% M and Delta must be square.
%
% let alfa=muub^2. For each possible value of matrix M scaling matrices
% D and G satisfy:
%
% M* D M + j (GM - M*G) <= alfa D
%
% where alfa is minimized. The LMI solver stops if muub becomes lower than target.
%
% If tol > 0 it corresponds to the tolerance, with which the LMI problem
% is solved (options(1) in gevp.m). Otherwise let epsilon = - tol. One imposes
% the additional constraint D > epsilon I.
%
% trace = 1 --> no display.
%
% Skew Mu Toolbox - G. Ferreres and J.M. Biannic, Onera-DCSD Toulouse, France.
%
if nargin==5
  M=a1;
  blk=a2;
  tol=a3;
  trace=a4;
  target=a5;
elseif nargin==6
  sys_M=a1;
  puls=a2;
  blk=a3;
  tol=a4;
  trace=a5;
  target=a6;
  M=freqresp(sys_M,puls);
else
  error('MU_UB: wrong number of input arguments');
end
%
% Preliminary tests
%
blk=blk(:,1:2);
%
idx=find(blk(:,1)<0);
if ~isempty(idx)
  qqq=blk(idx,2);
  if any(qqq~=0)
    error('MU_UB: full real blocks are not allowed');
  end
end
%
% The case of rectangular model perturbations is not accounted for.
%
if size(M,1)~=size(M,2) error('MU_UB: M must be square'); end
%
if ~square_delta(blk) error('MU_UB: Delta must be square'); end
%
nnn=sum(abs(blk(:,1)));
if size(M,1)~=nnn
  error('MU_UB: wrong respective sizes of M and Delta');
end
%
if tol>0
  options_gevp=tol;
  options_gevp(5)=trace;
  epsilon=0;
else
  epsilon=-tol;
  options_gevp=[];
  options_gevp(5)=trace;
end
%
Nbr_freq=size(M,3);
%
% mixed = 0 --> no real uncertainty, only complex ones.
%
mixed=any(blk(:,1)<0);
%
setlmis([]);
%
% Structure of D = Dr + j Di = D^{*}
%
% Structure of Dr = Dr^{T}
%
struct=[];
for i=1:size(blk,1)
  m=abs(blk(i,1));
  if blk(i,2)
    struct=[struct; m 0];
  else
    struct=[struct; m 1];
  end
end
%
[Dr,n,sDr]=lmivar(1,struct);
%
% Structure of Di = - Di^{T}
%
struct=[];
for i=1:size(blk,1)
  m=abs(blk(i,1));
  if blk(i,2)
    struct=mdiag(struct,zeros(m));
  elseif m==1
    struct=mdiag(struct,zeros(m));
  else
    qqq=skewdec(m,n);
    struct=mdiag(struct,qqq);
    n=max(max(abs(qqq)));
  end
end
%
if all(all(struct==0))
  D=lmivar(3,mdiag(sDr,sDr));
else
  [Di,n,sDi]=lmivar(3,struct);
  D=lmivar(3,[sDr sDi; -sDi sDr]);
end
%
% Structure of G = Gr + j Gi = G^{*}
%
if mixed
%
% Structure of Gr = Gr^{T}
%
  struct=[];
  for i=1:size(blk,1)
    m=abs(blk(i,1));
    if blk(i,1)<0
      struct=[struct; m  1];
    else
      struct=[struct; m -1];
    end
  end
%
  [Gr,n,sGr]=lmivar(1,struct);
%
% Structure of Gi = - Gi^{T}
%
  struct=[];
  for i=1:size(blk,1)
    m=abs(blk(i,1));
    if blk(i,1)>=0
      struct=mdiag(struct,zeros(m));
    elseif m==1
      struct=mdiag(struct,zeros(m));
    else
      qqq=skewdec(m,n);
      struct=mdiag(struct,qqq);
      n=max(max(abs(qqq)));
    end
  end
%
  if all(all(struct==0))
    G=lmivar(3,mdiag(sGr,sGr));
  else
    [Gi,n,sGi]=lmivar(3,struct);
    G=lmivar(3,[sGr sGi; -sGi sGr]);
  end
%
end
%
% D > epsilon I
%
lmiterm([-1 1 1 D],1,1);
lmiterm([ 1 1 1 0],epsilon);
icpt_lmi=2;
%
for iw=1:Nbr_freq
%
  H=M(:,:,iw);
  jH=sqrt(-1)*H;
  jH=[real(jH) imag(jH); -imag(jH) real(jH)];
  H =[real(H)   imag(H);  -imag(H)  real(H)];
%
% H* D H + G * jH + jH' * G < alfa^2 D
%
  lmiterm([icpt_lmi 1 1 D],H',H);
  if mixed
    lmiterm([icpt_lmi 1 1 G],1,jH,'s');
  end
  lmiterm([-icpt_lmi 1 1 D],1,1);
  icpt_lmi=icpt_lmi+1;
%
end
%
nbr_lmi_gevp=Nbr_freq;
%
LMIs=getlmis;
[tmin,xopt]=gevp(LMIs,nbr_lmi_gevp,options_gevp,[],[],target^2);
%
if isempty(xopt)
  fprintf('\n\nMU_UB: the LMI solver has failed (1)!\n\n');
  muu=Inf;
  D=NaN*ones(nnn);
  G=D;
  return;
end
%
muu=sqrt(tmin);
D=dec2mat(LMIs,xopt,D);
%
n=size(M,1);
%
if size(D,2)~=(2*n)
  fprintf('\n\nMU_UB: the LMI solver has failed (2)!\n\n');
  muu=Inf;
  D=NaN*ones(nnn);
  G=D;
  return;
end
Dr=D(1:n,1:n);
Di=D(1:n,(n+1):(2*n));
D=Dr+sqrt(-1)*Di;
%
if mixed
  G=dec2mat(LMIs,xopt,G);
  if size(G,2)~=(2*n)
    fprintf('\n\nMU_UB: the LMI solver has failed (3)!\n\n');
    muu=Inf;
    D=NaN*ones(nnn);
    G=D;
  return;
  end
  Gr=G(1:n,1:n);
  Gi=G(1:n,(n+1):(2*n));
  G=Gr+sqrt(-1)*Gi;
else
  G=zeros(size(D));
end


