function [muu,prt]=mu_zd(M,blk)
%
% call:  [muub,prt]=mu_zd(M,blk)
%
% Computation of a mu upper bound muub for complex matrix M and
% model perturbation Delta, whose structure is described by blk. Delta
% can only contain non-repeated real uncertainties. The method is
% exponential-time, and thus limited to about 10 or 12 uncertain
% parameters.
%
% The third column of blk describes the potentially skew or
% one-sided skew nature of the uncertainty: 
%
% blk(i,3) = 0 --> uncertain parameter # i maintained inside [-1,1]
%          = 1 --> uncertain parameter # i inside [-k,k], where k is maximized.
%          = 2 --> uncertain parameter # i inside [0,k],  where k is maximized.
%
% Internal use of a global variable "TAB_ZD"
%
% If matrix M is real prt provides the worst case model perturbation,
% since in this special case the mu upper bound coincides with the exact value.
% Otherwise, prt = NaN*ones(size(M));
%
% See for further details:
% R.R.E. De Gaston and M.G. Safonov,
% "Exact calculation of the multiloop stability margin",
% IEEE Transactions on Automatic Control, vol. 33, n. 2, pp. 156-171, 1988.
%
% Skew Mu Toolbox - G. Ferreres and J.M. Biannic, Onera-DCSD Toulouse, France.
%
if nargin~=2 error('MU_ZD: two inputs arguments'); end
%
if any(blk(:,1)~=-1)|any(blk(:,2)~=0)
  error('MU_ZD: only non-repeated real uncertainties are allowed');
end
%
% tol = relative tolerance for the dichotomy search
%
% if the mu upper bound is less than mu_min, the mu upper bound is limited to mu_min
%
tol=5e-2;
mu_min=1e-3;
%
qqq=norm(imag(M))/norm(M);
real_case=(qqq<=1e-10);
%
% To compute the worst-case perturbation with a satisfactory accuracy
%
if real_case tol=1e-4; end
%
Kmax=1/mu_min;
%
% Computation of TAB_ZD, which contains the vertices of the unit hypercube
%
global TAB_ZD;
n=size(M,1);
nn=size(TAB_ZD,1);
if (nn~=2^n) TAB_ZD=hypercube(n); end
%
% Preliminary test: mu < mu_min ?
%
tab=zeros(size(TAB_ZD));
for i=1:size(TAB_ZD,1)
  delta=TAB_ZD(i,:);
  for j=1:length(delta)
    if blk(j,3)==1
      delta(j)=Kmax*delta(j);
    elseif blk(j,3)==2
      if delta(j)==-1
        delta(j)=0;
      else
        delta(j)=Kmax;
      end
    end
  end
  tab(i,:)=delta;
end
[ok,prt2]=mu_ZD_bis(M,tab);
if ok
  muu=1/Kmax;
  prt=NaN*ones(size(M));
  return;
else
  prt=prt2;
end
%
% Dichotomy search to compute the lowest mu upper bound
%
Kmin=0;
the_end=0;
while ~the_end
%
  K=(Kmin+Kmax)/2;
%
  tab=zeros(size(TAB_ZD));
  for i=1:size(TAB_ZD,1)
    delta=TAB_ZD(i,:);
    for j=1:length(delta)
      if blk(j,3)==1
        delta(j)=K*delta(j);
      elseif blk(j,3)==2
        if delta(j)==-1
          delta(j)=0;
        else
          delta(j)=K;
        end
      end
    end
    tab(i,:)=delta;
  end
%
  [ok,prt2]=mu_ZD_bis(M,tab);
  if ok Kmin=K; else Kmax=K; prt=prt2; end
  gap=(Kmax-Kmin)/Kmax;
  the_end=(gap<=tol);
%
end
%
muu=1/Kmin;
%
if ~real_case
  prt=NaN*ones(size(M));
end
%
% *****************************************************************
%
function [ok,prt_star]=mu_ZD_bis(M,TAB)
%
% call:  [ok,prt_star]=mu_ZD_bis(M,TAB)
%
% ok = 1 --> the origin of the complex plane does not belong to the convex hull
%
% May 03
%
if nargin~=2 error('MU_ZD_BIS: two input arguments'); end
%
% Computation of images of vertices of the hypercube in the complex plane
%
p=zeros(size(TAB,1),1);
%
for i=1:size(TAB,1)
  p(i)=det(eye(size(M))-M*diag(TAB(i,:)));
end
%
[qqq,i]=min(abs(p));
prt_star=diag(TAB(i,:));
%
qqq=norm(imag(M))/norm(M);
if qqq<=1e-10
%
% M is a real matrix
%
  p=real(p);
  q1=min(p);
  q2=max(p);
  ok=(sign(q1)==sign(q2));
%
else
%
% M is a complex matrix
%
  qqq=(imag(p)>=0);
  list1=find(qqq);
  list2=find(~qqq);
  if (~isempty(list1))&(~isempty(list2))
    p1=p(list1);
    p2=p(list2);
    [q1,i1]=min(real(p1)./abs(p1));
    [q2,i2]=min(real(p2)./abs(p2));
    i1=list1(i1);
    i2=list2(i2);
    x1=real(p(i1));
    y1=imag(p(i1));
    x2=real(p(i2));
    y2=imag(p(i2));
    x=-(x2-x1)/(y2-y1)*y1+x1;
    ok=(x>0);
  else
    index=(imag(p)==0);
    if isempty(index)
      ok=1;
    else
      qqq=real(p(index));
      q1=min(qqq);
      q2=max(qqq);
      ok=(sign(q1)==sign(q2));
    end
  end
%
end
