function robust(sys_M,blk,puls,truncated_sector)
%
% calls: robust(sys_M,blk)
%        robust(sys_M,blk,puls)
%        robust(sys_M,blk,puls,truncated_sector)
%        robust(sys_M,blk,[],truncated_sector)
%
% Robustness analysis of the standard interconnection structure M(s) - Delta.
%
% sys_M is a system matrix of M(s) with ss format.
%
% blk describes the structure of the model perturbation Delta.
% The 3d column describes the potentially skew nature of the uncertainties.
% blk(i,3) = 0 --> norm of the uncertainty less than 1.
%          = 1 --> norm of the uncertainty less than k, where k is maximized.
%
% Only SQUARE performance blocks and neglected dynamics are allowed.
% (and possibly repeated real or complex scalars)
%
% puls = initial frequency gridding (it may be refined inside the routine).
%
% Description of the truncated sector:
% truncated_sector(1) = alfa_min
% truncated_sector(2) = cxi_min
% alfa_min = minimal degree of stability (negative for a stable closed loop)
% (degree of stability = maximal real part of the poles)
% cxi_min = minimal value of the damping ratio.
% Neglected dynamics or performance blocks are not defined outside the imaginary axis.
% Thus only parametric uncertainties are allowed when studying robust stability inside
% a truncated sector.
%
% If puls=[] and/or truncated_sector=[], corresponding values will be asked for
% in the routine.
%
% The mu method is asked for inside the routine.
%
% Skew Mu Toolbox - G. Ferreres and J.M. Biannic, Onera-DCSD Toulouse, France.
%
if nargin==2
  puls=[];
  truncated_sector=[];
elseif nargin==3
  truncated_sector=[];
elseif nargin~=4
  error('ROBUST: wrong number of input arguments');
end
%
% Preliminary tests
%
if size(blk,2)~=3 error('ROBUST: blk must contain 3 columns'); end
%
if isempty(puls)
  rep=input2('Frequency gridding generated automatically (1) or "manually" (0): ',1);
  if rep
    puls=gen_grid(sys_M);
  else
    fprintf('\n\n Computation of the initial freq. gridding between wmin (a non-zero value which may be very small) and wmax\n\n');
    the_end=0;
    while ~the_end
      wmin=input2('wmin = ',[]);
      the_end=(wmin>0);
    end
    wmax=input2('wmax = ',[]);
    N=input2('Number of logarithmically spaced frequency points: ',100);
    puls=logspace2(wmin,wmax,N);
    rep=input2('Do you want to add the zero frequency (0/1): ',1);
    if rep
      puls=[0 puls];
      fprintf('\n*** The zero frequency has been added ***\n');
    end
    rep=input2('Do you want to refine the gridding near the bending modes frequencies (if any - 0/1): ',0);
    if rep
      puls2=calc_freq_gridding_bending(sys_M);
      puls=[puls puls2];
      puls=sort(puls);
      rep=input2('Do you want to eliminate nearly identical frequency points (0/1): ',0);
      if rep==1
        puls_new=puls(1);
        for i=2:length(puls)
          if abs(puls(i)-puls_new(end))>1e-4*puls(i)    % the threshold 1e-4 may be adjusted
            puls_new=[puls_new puls(i)];
          end
        end
        fprintf('\n\nSize of the old and new frequency griddings: %i and %i\n\n',length(puls),length(puls_new));
        puls=puls_new;
      end
    end
  end 
end
%
puls0=puls;
%
% mixed = 0 --> only parametric uncertainties in Delta.
%
mixed=any(blk(:,1)>=0);
%
% skew_mu_problem = 0 --> classical mu problem
%
skew_mu_problem=any(blk(:,3)==0);
%
if isempty(truncated_sector)
  if mixed
    alfa_min=0;
    cxi_min=0;
    fprintf('\n\n Robust stability inside the Left Half Plane\n\n');
  else
    fprintf('\n\n Poles of the nominal closed loop (which must be inside the specified truncated sector)\n\n');
    damp(sys_M);
    alfa_min=input2('Required degree of stability (negative for a stable closed loop): ',0);
    cxi_min =input2('Required minimal damping ratio: ',0);
  end
else
  alfa_min=truncated_sector(1);
  cxi_min =truncated_sector(2);
  if mixed
    if (alfa_min~=0)|(cxi_min~=0)
      error('ROBUST: neglected dynamics and/or performance blocks are not defined outside the imaginary axis');
    end
  end
end
%
[w,z,e]=damp(sys_M);
z=min(z);
if (max(real(e))>=alfa_min)|(z<=cxi_min)
  error('ROBUST: Poles of the nominal closed loop are not inside the specified truncated sector');
end
%
for i=1:3, figure(i); clf; end
first_time=1;
tab_c='mcrgby';
idx_tab_c=1;
%
LHP=(alfa_min==0)&(cxi_min==0);
%
the_end=0;
while ~the_end
%
  if LHP
    rep=menu('computational method',....
    'Use of a frequency gridding',...
    'Computation of a mu lower bound with a freq. gridding and a state-space approach (only for real uncertainties - no skew unc.)',...
    'Computation of a mu upper bound on a union of frequency intervals',...
    'Elimination of frequency intervals (mu test)',...
    'Computation of a mu upper bound by frequency sweeping (mu_max_2)',...
    'End');
  else
    rep=menu('computational method',....
    'Use of a frequency gridding',...
    'Computation of a mu lower bound with a frequency gridding and a state-space approach (no skew uncertainty)',...
    'Computation of a mu upper bound on a union of frequency intervals',...
    'End');
    if rep==4 rep=6; end
  end
%
% *** Use of a frequency gridding ***
%
  if rep==1
%
    non_repeated_real_scalars=all(blk(:,1)==-1);
%
    if mixed|(~non_repeated_real_scalars)
      method=menu('Choice of the mu method',...
      'mixed mu upper bound',...
      'mixed mu lower bound');
    else
      method=menu('Choice of the mu method',...
      'mixed mu upper bound',...
      'mixed mu lower bound',...
      'mu upper bound (Zadeh and Desoer - nonrepeated real uncertainties)',...
      'mu lower bound (Dailey - nonrepeated real uncertainties)');
    end
%
    puls=puls0;
    [tab_mu,tab_pert]=mu_frequency_gridding(sys_M,blk,puls,method,alfa_min,cxi_min);
    [qqq,i]=max(tab_mu);
    if (puls(i)~=0)&(method~=2)
      puls_bis=logspace2(0.8*puls(i),1.2*puls(i),30); % May be adjusted
      [tab_mu_bis,tab_pert_bis]=mu_frequency_gridding(sys_M,blk,puls_bis,method,alfa_min,cxi_min);
      puls=[puls puls_bis];
      tab_mu=[tab_mu tab_mu_bis];
      [puls,idx]=sort(puls);
      tab_mu=tab_mu(idx);
      if ~isempty(tab_pert)
        nnn=size(tab_pert,3);
        for iii=1:size(tab_pert_bis,3)
          tab_pert(:,:,iii+nnn)=tab_pert_bis(:,:,iii);
        end
        tab_pert=tab_pert(:,:,idx);
      end
    end
%
    if puls(1)==0
      fprintf('\n mu = %6.3f at w = 0\n',tab_mu(1));
    end
    [mu_max,iii]=max(tab_mu);
    omega_max=puls(iii);
    fprintf(' Maximal value of mu = %6.3f at w = %6.3f rad/s\n',mu_max,omega_max);
%
    figure(1);
    if first_time first_time=0; else hold on; end
    semilogx(puls,tab_mu,tab_c(idx_tab_c));
    grid on;
    xlabel('rad/s');
    ylabel('mu');
    hold off;
    if method==1
      fprintf(['*** ',tab_c(idx_tab_c),': mixed mu upper bound\n']);
    elseif method==2
      fprintf(['*** ',tab_c(idx_tab_c),': mixed mu lower bound\n']);
    elseif method==3
      fprintf(['*** ',tab_c(idx_tab_c),': mu upper bound (Zadeh and Desoer)\n']);
    elseif method==4
      fprintf(['*** ',tab_c(idx_tab_c),': mu lower bound (Dailey)\n']);
    end
    idx_tab_c=idx_tab_c+1;
    if idx_tab_c>length(tab_c) idx_tab_c=1; end
%
    if (method==2)|(method==4)
      [mu_max,iii]=max(tab_mu);
      draw_locus=input2('Draw of the root locus, associated to the smallest destabilizing model perturbation (0/1): ',0);
      if draw_locus
        figure(2);
        clf;
        mu_locus(sys_M,tab_pert(:,:,iii),'r');
        sgrid(0:0.1:1,[]);
        visu_sector(2,alfa_min,cxi_min,'b');
        zoom on;
        fprintf(' Zoom is activated on figure 2\n');
      end
    elseif (~mixed)&(~skew_mu_problem)
      rep2=input2('Do you want to compute a mu lower bound around the critical frequency (0/1): ',0);
      if rep2==1
        if omega_max==0
          puls=0;
        else
          puls=linspace(0.9*omega_max,1.1*omega_max,10);
        end
        robust_mu_lb;
      end
    end
%
% *** Computation of a mu lower bound with a freq. gridding and a state-space approach ***
%
  elseif rep==2
    if mixed
      fprintf('\n\n The method is limited to parametric uncertainties\n\n');
    elseif skew_mu_problem
      fprintf('\n\n The method is limited to classical mu problems\n\n');
    else
      lgpuls0=length(puls0);
      fprintf('\n\n Present frequency gridding: %i points between %6.3f rad/s and %6.3f rad/s\n',...
      lgpuls0,min(puls0),max(puls0));
      if lgpuls0 > 25
        fprintf(' It''s generally useless to use a fine frequency gridding with this method....');
        fprintf('\n The size of the frequency gridding is divided by N (an integer)\n');
        Ndefault=round(lgpuls0/25);
        N=input2('N = ',Ndefault);
        idx=1:N:lgpuls0;
        puls=puls0(idx);
      end
      robust_mu_lb;
    end
%
% *** Computation of a mu upper bound on a union of frequency intervals ***
%
  elseif rep==3
%
    lgpuls0=length(puls0);
    fprintf('\n\n Present frequency gridding: %i points between %6.3f rad/s and %6.3f rad/s\n',...
    lgpuls0,min(puls0),max(puls0)); 
    if lgpuls0 > 5
        fprintf(' It''s generally useless to use a fine frequency gridding with this method....');
        fprintf('\n The size of the frequency gridding is divided by N (an integer)\n');
        Ndefault=round(lgpuls0/5);
        N=input2('N = ',Ndefault);
        idx=1:N:lgpuls0;
        puls=puls0(idx);
        if puls(end)~=puls0(end) puls=[puls puls0(end)]; end
    end
%
    robust_mu_ub;
    fprintf(['*** ',tab_c(idx_tab_c),': mu upper bound on a union of frequency intervals\n\n']);
    idx_tab_c=idx_tab_c+1;
    if idx_tab_c>length(tab_c) idx_tab_c=1; end
%
    if (~mixed)&(~skew_mu_problem)
      rep2=input2('Do you want to compute a mu lower bound on the critical frequency interval (0/1): ',1);
      if rep2==1
        puls=linspace(w1,w2,10);
        robust_mu_lb;
      end
    end
%
% *** Elimination of frequency intervals (mu test) ***
%
  elseif rep==4
%
    fprintf('\n\nThe issue is to check whether (skew) mu <= gamma inside [wmin, wmax]\n\n');
    gamma=input2('gamma: ',[]);
    wmin=input2('wmin: ',[]);
    wmax=input2('wmax (not Inf): ',[]);
    nbr_iter_max=input2('Maximal number of iterations: ',5);
    fprintf('\ndefault value is ''c''\n');
    opt=input('option of mu.m: ','s');
    if isempty(opt) opt='c'; end
    fprintf('\nIf a large number of iterations is chosen, stop if mu > gamma is advised.');
    possible_stop=input2('stop if mu > gamma is found (0/1): ',0);
    epsilon=input2('Quantity of additional complex uncertainties (0.01 means 1 percent): ',0);
    verbatim=input2('Display of intermediate results (0/1): ',1);
    [list_intervals,list_intervals_OK,critical_pts]=...
    mu_elim(sys_M,blk,gamma,[wmin wmax],nbr_iter_max,opt,possible_stop,epsilon,verbatim);
    if ~isempty(list_intervals_OK)
      fprintf('\n List of intervals inside which (skew) mu <= %6.3f is guaranteed:\n',gamma);
      for i=1:size(list_intervals_OK,1)
        fprintf('%6.3f - %6.3f rad/s\n',list_intervals_OK(i,1),list_intervals_OK(i,2));
      end
    end
    if ~isempty(list_intervals)
      fprintf('\n List of intervals inside which (skew) mu <= %6.3f is NOT guaranteed:\n',gamma);
      for i=1:size(list_intervals,1)
        fprintf('%6.3f - %6.3f rad/s\n',list_intervals(i,1),list_intervals(i,2));
      end
    end
    if ~isempty(critical_pts)
      fprintf('\n List of freq., at which the (skew) mu upper bound is greater than %6.3f:\n',gamma);
      disp(critical_pts);
    end
%
% *** Computation of a mu upper bound by frequency sweeping (mu_max_2) ***
%
  elseif rep==5
%
    fprintf('\n\n Present frequency gridding: %i points between %6.3f rad/s and %6.3f rad/s\n',...
    length(puls0),min(puls0),max(puls0));
    fprintf(' It''s time consuming to use a fine frequency gridding with this method....');
    fprintf('\n The size of the frequency gridding is divided by N (an integer)\n');
    N=input2('N = ',20);
    idx=1:N:length(puls0);
    puls=puls0(idx);
    if puls(end)~=puls0(end) puls=[puls puls0(end)]; end
%
    [tab_beta_visu,tab_puls_visu]=mu_max_2(sys_M,blk,puls);
    [beta,i]=max(tab_beta_visu);
    w1=tab_puls_visu(i,1);
    w2=tab_puls_visu(i,2);
    fprintf('\n\n maximal value of the mu upper bound = %6.3f between %6.3f rad/s and %6.3f rad/s \n\n',beta,w1,w2);
    figure(1);
    if first_time first_time=0; else hold on; end
    plot_bar(tab_puls_visu,tab_beta_visu,'lin',tab_c(idx_tab_c));
    xlabel('rad/s');
    ylabel('mu');
    hold off;
    fprintf(['*** ',tab_c(idx_tab_c),': mu upper bound on a union of frequency intervals (state-space approach)\n\n']);
    idx_tab_c=idx_tab_c+1;
    if idx_tab_c>length(tab_c) idx_tab_c=1; end
%
    if (~mixed)&(~skew_mu_problem)
      rep2=input2('Do you want to compute a mu lower bound on the critical frequency interval (0/1): ',1);
      if rep2==1
        puls=linspace(w1,w2,10);
        robust_mu_lb;
      end
    end
%
  elseif rep==6
    the_end=1;
  end
%
end
