function muu=skew_mu_ub(a1,a2,a3,a4,a5,a6)
%
% calls: muub=skew_mu_ub(M,blk,tol,trace,target)
%        muub=skew_mu_ub(sys_M,puls,blk,tol,trace,target)
%
% Computation of a skew mixed mu upper bound,
% which is simultaneously valid for several values of matrix M.
%
% These values may correspond to  M(:,:,i) for i between 1 and N (1st call),
% or to the frequency response of system matrix sys_M (ss format)
% on the frequency gridding puls (2d call).
%
% The structure of the model perturbation Delta is described by blk.
% The third column describes the potentially skew nature of
% the uncertainty:
%
% blk(i,3) = 0 --> norm of the uncertainty less than 1 (skew uncertainty).
%          = 1 --> norm of the uncertainty less than k, where k is maximized.
%
% If blk(i,3)=1 for all i (i.e. a classical mu problem, not a skew mu one),
% mu_ub.m is called inside the routine.
%
% M and Delta must be square.
%
% The LMI solver stops if the upper bound becomes lower than target.
%
% If tol > 0 it corresponds to the tolerance, with which the LMI problem
% is solved (options(1) in gevp.m). Otherwise, let epsilon = - tol. One imposes
% additional constraints of the type D > epsilon I.
%
% trace = 1 --> no display.
%
% See also skew_mu_ub_bis.m
%
% Skew Mu Toolbox - G. Ferreres and J.M. Biannic, Onera-DCSD Toulouse, France.
%
if nargin==5 
  M=a1;
  blk=a2;
  tol=a3;
  trace=a4;
  target=a5;
elseif nargin==6
  sys_M=a1;
  puls=a2;
  blk=a3;
  tol=a4;
  trace=a5;
  target=a6;
  M=freqresp(sys_M,puls);
else
  error('SKEW_MU_UB: wrong number of input arguments');
end
%
% Preliminary tests
%
if size(blk,2)~=3 error('SKEW_MU_UB: blk must have 3 columns'); end
%
if all(blk(:,3)==0) error('SKEW_MU_UB: problem with the 3th column of blk'); end
%
idx=find(blk(:,1)<0);
if ~isempty(idx)
  qqq=blk(idx,2);
  if any(qqq~=0)
    error('SKEW_MU_UB: full real blocks are not allowed');
  end
end
%
% The case of rectangular model perturbations is not accounted for.
%
if size(M,1)~=size(M,2) error('SKEW_MU_UB: M must be square'); end
%
if ~square_delta(blk) error('SKEW_MU_UB: Delta must be square'); end
%
nnn=sum(abs(blk(:,1)));
if size(M,1)~=nnn
  error('SKEW_MU_UB: wrong respective sizes of M and Delta');
end
%
% Case of a classical mu problem (no skew uncertainty)
%
if all(blk(:,3)==1)
  muu=mu_ub(M,blk,tol,trace,target);
  return;
end
%
[M2,blk2]=skew_sort(M,blk,0);
%
muu=skew_mu_ub_bis(M2,blk2,tol,trace,target);
