function [muu,H,D,G]=skew_mu_ub_bis(a1,a2,a3,a4,a5,a6)
%
% calls: [muub,H,D,G]=skew_mu_ub_bis(M,blk,tol,trace,target)
%        [muub,H,D,G]=skew_mu_ub_bis(sys_M,puls,blk,tol,trace,target)
%
% Computation of a skew mixed mu upper bound muub,
% which is simultaneously valid for several values of matrix M.
%
% These values may correspond to  M(:,:,i) for i between 1 and N (1st call),
% or to the frequency response of system matrix sys_M (ss format)
% on the frequency gridding puls (2d call).
%
% The structure of the model perturbation Delta is described by blk.
% The third column describes the potentially skew nature of
% the uncertainty:
%
% blk(i,3) = 0 --> norm of the uncertainty less than 1 (skew uncertainty).
%          = 1 --> norm of the uncertainty less than k, where k is maximized.
%
% If blk(i,3)=1 for all i (i.e. a classical mu problem, not a skew mu one),
% mu_ub.m is called inside the routine.
%
% M and Delta must be square.
%
% Delta must have the form diag(Delta1,Delta2)
% where Delta1 is maintained inside the unit ball
% while the size of Delta2 is free.
% See skew_sort.m for such a transformation.
% See also skew_mu_ub.m.
%
% The LMI solver stops if the upper bound becomes lower than target.
%
% If tol > 0 it corresponds to the tolerance, with which the LMI problem
% is solved (options(1) in gevp.m). Otherwise, let epsilon = - tol. One imposes
% additional constraints of the type D > epsilon I.
%
% trace = 1 --> no display.
%
% The output arguments H,D,G satisfy the LMI:
%
% M2* D M2 + j (G M2 - M2* G) <= muub*muub D
%
% where M2 = H * M and
%% 
%     | muub I  0 |
% H = |           | according to Delta = diag(Delta1,Delta2)
%     | 0       I |
%
% Last modification: 07/26/04
% mincx is called instead of gevp for a robust performance problem.
%
% Skew Mu Toolbox - G. Ferreres and J.M. Biannic, Onera-DCSD Toulouse, France.
%
if nargin==5 
  M=a1;
  blk=a2;
  tol=a3;
  trace=a4;
  target=a5;
elseif nargin==6
  sys_M=a1;
  puls=a2;
  blk=a3;
  tol=a4;
  trace=a5;
  target=a6;
  M=freqresp(sys_M,puls);
else
  error('SKEW_MU_UB_BIS: wrong number of input arguments');
end
%
% Preliminary tests
%
if size(blk,2)~=3 error('SKEW_MU_UB_BIS: blk must have 3 columns'); end
%
if all(blk(:,3)==0) error('SKEW_MU_UB_BIS: problem with the 3th column of blk'); end
%
idx=find(blk(:,1)<0);
if ~isempty(idx)
  qqq=blk(idx,2);
  if any(qqq~=0)
    error('SKEW_MU_UB_BIS: full real blocks are not allowed');
  end
end
%
% The case of rectangular model perturbations is not accounted for.
%
if size(M,1)~=size(M,2) error('SKEW_MU_UB_BIS: M must be square'); end
%
if ~square_delta(blk) error('SKEW_MU_UB_BIS: Delta must be square'); end
%
nnn=sum(abs(blk(:,1)));
if size(M,1)~=nnn
  error('SKEW_MU_UB_BIS: wrong respective sizes of M and Delta');
end
%
% Case of a classical mu problem (no skew uncertainty)
%
if all(blk(:,3)==1)
  [muu,D,G]=mu_ub(M,blk,tol,trace,target);
  H=eye(size(M(:,:,1)));
  return;
end
%
if tol>0
  options_gevp=tol;
  options_gevp(5)=trace;
  epsilon=0;
else
  epsilon=-tol;
  options_gevp=[];
  options_gevp(5)=trace;
end
%
% Check whether Delta = diag(Delta1,Delta2)
%
aaa=blk(:,3);
aaa2=sort(aaa);
if any(aaa~=aaa2)
  error('SKEW_MU_UB_BIS: Delta is not sorted as required');
end
%
list=find(aaa==0);
m1=max(list);
idx_blk1=1:m1;
idx_blk2=(m1+1):length(aaa);
blk1=blk(idx_blk1,1:2);
blk2=blk(idx_blk2,1:2);
N1=sum(abs(blk1(:,1)));
N2=sum(abs(blk2(:,1)));
idx_Delta1=1:N1;
idx_Delta2=(N1+1):(N1+N2);
%
robust_performance=0;
if size(blk2,1)==1
  qqq1=blk2(1);
  qqq2=blk2(2);
  if qqq1==1 qqq2=1; end
  robust_performance=(qqq1>0)&(qqq2==qqq1);
end
%
Nbr_freq=size(M,3);
%
M11=M(idx_Delta1,idx_Delta1,:);
M12=M(idx_Delta1,idx_Delta2,:);
M21=M(idx_Delta2,idx_Delta1,:);
M22=M(idx_Delta2,idx_Delta2,:);
%
% Preliminary test: elimination of the case where the skew mu upper bound is infinite
%
muu=mu_ub(M11,blk1,tol,trace,0.99);
if muu>=1
  muu=Inf;
  H=NaN*ones(nnn);
  D=H;
  G=H;
  return;
end
%
% mixed_1 = 0 --> no real uncertainty, only complex ones in Delta1.
%
mixed_1=any(blk1(:,1)<0);
%
% mixed_2 = 0 --> no real uncertainty, only complex ones in Delta2.
%
mixed_2=any(blk2(:,1)<0);
%
setlmis([]);
%
% Structure of D1 = D1r + j D1i = D1^{*}
%
% Structure of D1r
%
struct=[];
for i=1:size(blk1,1)
  m=abs(blk1(i,1));
  if blk1(i,2)
    struct=[struct; m 0];
  else
    struct=[struct; m 1];
  end
end
%
[D1r,n,sD1r]=lmivar(1,struct);
%
% Structure of D1i
%
struct=[];
for i=1:size(blk1,1)
  m=abs(blk1(i,1));
  if blk1(i,2)
    struct=mdiag(struct,zeros(m));
  elseif m==1
    struct=mdiag(struct,zeros(m));
  else
    qqq=skewdec(m,n);
    struct=mdiag(struct,qqq);
    n=max(max(abs(qqq)));
  end
end
%
if all(all(struct==0))
  D1=lmivar(3,mdiag(sD1r,sD1r));
else
  [D1i,n,sD1i]=lmivar(3,struct);
  D1=lmivar(3,[sD1r sD1i; -sD1i sD1r]);
end
%
% Structure of D2 = D2r + j D2i = D2^{*}
%
% Structure of D2r
%
if ~robust_performance
%
  struct=[];
  for i=1:size(blk2,1)
    m=abs(blk2(i,1));
    if blk2(i,2)
      struct=[struct; m 0];
    else
      struct=[struct; m 1];
    end
  end
%
  [D2r,n,sD2r]=lmivar(1,struct);
%
% Structure of D2i
%
  struct=[];
  for i=1:size(blk2,1)
    m=abs(blk2(i,1));
    if blk2(i,2)
      struct=mdiag(struct,zeros(m));
    elseif m==1
      struct=mdiag(struct,zeros(m));
    else
      qqq=skewdec(m,n);
      struct=mdiag(struct,qqq);
      n=max(max(abs(qqq)));
    end
  end
%
  if all(all(struct==0))
    D2=lmivar(3,mdiag(sD2r,sD2r));
  else
    [D2i,n,sD2i]=lmivar(3,struct);
    D2=lmivar(3,[sD2r sD2i; -sD2i sD2r]);
  end
%
% Structure of Y (the same as D2)
%
  struct=[];
  for i=1:size(blk2,1)
    m=abs(blk2(i,1));
    if blk2(i,2)
      struct=[struct; m 0];
    else
      struct=[struct; m 1];
    end
  end
%
  [Yr,n,sYr]=lmivar(1,struct);
%
% Structure of Yi
%
  struct=[];
  for i=1:size(blk2,1)
    m=abs(blk2(i,1));
    if blk2(i,2)
      struct=mdiag(struct,zeros(m));
    elseif m==1
      struct=mdiag(struct,zeros(m));
    else
      qqq=skewdec(m,n);
      struct=mdiag(struct,qqq);
      n=max(max(abs(qqq)));
    end
  end
%
  if all(all(struct==0))
    Y=lmivar(3,mdiag(sYr,sYr));
    sYi=zeros(size(sYr));
  else
    [Yi,n,sYi]=lmivar(3,struct);
    [Y,n,struct_Y]=lmivar(3,[sYr sYi; -sYi sYr]);
  end
%
else
  m=blk2(1,1);
  [Y,n_variable_to_be_minimised]=lmivar(1,[2*m 0]);
end
%
% Structure of G1 = = G1r + j G1i = G1^{*}
%
if mixed_1
%
% Structure of G1r
%
  struct=[];
  for i=1:size(blk1,1)
    m=abs(blk1(i,1));
    if blk1(i,1)<0
      struct=[struct; m  1];
    else
      struct=[struct; m -1];
    end
  end
%
  [G1r,n,sG1r]=lmivar(1,struct);
%
% Structure of G1i
%
  struct=[];
  for i=1:size(blk1,1)
    m=abs(blk1(i,1));
    if blk1(i,1)>=0
      struct=mdiag(struct,zeros(m));
    elseif m==1
      struct=mdiag(struct,zeros(m));
    else
      qqq=skewdec(m,n);
      struct=mdiag(struct,qqq);
      n=max(max(abs(qqq)));
    end
  end
%
  if all(all(struct==0))
    G1=lmivar(3,mdiag(sG1r,sG1r));
  else
    [G1i,n,sG1i]=lmivar(3,struct);
    G1=lmivar(3,[sG1r sG1i; -sG1i sG1r]);
  end
%
end
%
% Structure of G2 = G2r + j G2i = G2^{*}
%
if mixed_2
%
% Structure of G2r
%
  struct=[];
  for i=1:size(blk2,1)
    m=abs(blk2(i,1));
    if blk2(i,1)<0
      struct=[struct; m  1];
    else
      struct=[struct; m -1];
    end
  end
%
  [G2r,n,sG2r]=lmivar(1,struct);
%
% Structure of G2i
%
  struct=[];
  for i=1:size(blk2,1)
    m=abs(blk2(i,1));
    if blk2(i,1)>=0
      struct=mdiag(struct,zeros(m));
    elseif m==1
      struct=mdiag(struct,zeros(m));
    else
      qqq=skewdec(m,n);
      struct=mdiag(struct,qqq);
      n=max(max(abs(qqq)));
    end
  end
%
  if all(all(struct==0))
    G2=lmivar(3,mdiag(sG2r,sG2r));
  else
    [G2i,n,sG2i]=lmivar(3,struct);
    G2=lmivar(3,[sG2r sG2i; -sG2i sG2r]);
  end
%
end
%
% D1 > epsilon I
%
lmiterm([-1 1 1 D1],1,1);
lmiterm([ 1 1 1 0],epsilon);
%
% D2 > epsilon I
%
if ~robust_performance
  lmiterm([-2 1 1 D2],1,1);
  lmiterm([ 2 1 1 0],epsilon); 
end
%
icpt_lmi=3;
%
for iw=1:Nbr_freq
%
  H11=[real(M11(:,:,iw)) imag(M11(:,:,iw)); -imag(M11(:,:,iw)) real(M11(:,:,iw))];
  H12=[real(M12(:,:,iw)) imag(M12(:,:,iw)); -imag(M12(:,:,iw)) real(M12(:,:,iw))];
  H21=[real(M21(:,:,iw)) imag(M21(:,:,iw)); -imag(M21(:,:,iw)) real(M21(:,:,iw))];
  H22=[real(M22(:,:,iw)) imag(M22(:,:,iw)); -imag(M22(:,:,iw)) real(M22(:,:,iw))];
%
  X11=sqrt(-1)*M11(:,:,iw);
  X12=sqrt(-1)*M12(:,:,iw);
  X21=sqrt(-1)*M21(:,:,iw);
  X22=sqrt(-1)*M22(:,:,iw);
  X11=[real(X11) imag(X11); -imag(X11) real(X11)];
  X12=[real(X12) imag(X12); -imag(X12) real(X12)];
  X21=[real(X21) imag(X21); -imag(X21) real(X21)];
  X22=[real(X22) imag(X22); -imag(X22) real(X22)];
%
  lmiterm([icpt_lmi 1 1 D1],H11',H11);
  if ~robust_performance
    lmiterm([icpt_lmi 1 1 D2],H21',H21);
  else
    lmiterm([icpt_lmi 1 1 0],H21'*H21);
  end
  lmiterm([-icpt_lmi 1 1 D1],1,1);
  if mixed_1
    lmiterm([icpt_lmi 1 1 G1],1,X11,'s');
  end
%
  lmiterm([icpt_lmi 1 2 D1],H11',H12);
  if ~robust_performance
    lmiterm([icpt_lmi 1 2 D2],H21',H22);
  else
    lmiterm([icpt_lmi 1 2 0],H21'*H22);
  end
  if mixed_1
    lmiterm([icpt_lmi 1 2 G1],1,X12);
  end
  if mixed_2
    lmiterm([icpt_lmi 1 2 G2],X21',1);
  end
%
  if ~robust_performance  
    lmiterm([icpt_lmi 2 2 D2],H22',H22);
  else
    lmiterm([icpt_lmi 2 2 0],H22'*H22);
  end
  lmiterm([icpt_lmi 2 2 D1],H12',H12);
  lmiterm([-icpt_lmi 2 2 Y],1,1);
  if mixed_2
    lmiterm([icpt_lmi 2 2 G2],1,X22,'s');
  end
%
  icpt_lmi=icpt_lmi+1;
%
end
%
% Y < alfa D2, where alfa is minimized
%
if ~robust_performance  
  lmiterm([ icpt_lmi 1 1  Y],1,1);
  lmiterm([-icpt_lmi 1 1 D2],1,1);
  nbr_lmi_gevp=1;
  LMIs=getlmis;
  [tmin,xopt]=gevp(LMIs,nbr_lmi_gevp,options_gevp,[],[],target^2);
else
  LMIs=getlmis;
  n=decnbr(LMIs);
  c=zeros(1,n);
  c(n_variable_to_be_minimised)=1;
  [tmin,xopt]=mincx(LMIs,c,options_gevp,[],target^2);
end
%
if isempty(xopt)
  fprintf('\n\nSKEW_MU_UB_BIS: the LMI solver has failed (1)!\n\n');
  muu=Inf;
  H=NaN*ones(nnn);
  D=H;
  G=H;
  return;
end
%
muu=sqrt(tmin);
%
D1=dec2mat(LMIs,xopt,D1);
if ~robust_performance
  D2=dec2mat(LMIs,xopt,D2);
else
  m=blk2(1,1);
  D2=eye(2*m);
end
%
n1=size(M11,1);
if size(D1,2)~=(2*n1)
  fprintf('\n\nSKEW_MU_UB_BIS: the LMI solver has failed (2)!\n\n');
  muu=Inf;
  H=NaN*ones(nnn);
  D=H;
  G=H;
  return;
end
Dr=D1(1:n1,1:n1);
Di=D1(1:n1,(n1+1):(2*n1));
D1=Dr+sqrt(-1)*Di;
%
if mixed_1
  G1=dec2mat(LMIs,xopt,G1);
  if size(G1,2)~=(2*n1)
    fprintf('\n\nSKEW_MU_UB_BIS: the LMI solver has failed (3)!\n\n');
    muu=Inf;
    H=NaN*ones(nnn);
    D=H;
    G=H;
    return;
  end
  Gr=G1(1:n1,1:n1);
  Gi=G1(1:n1,(n1+1):(2*n1));
  G1=Gr+sqrt(-1)*Gi;
else
  G1=zeros(size(D1));
end
%
n2=size(M22,1);
if size(D2,2)~=(2*n2)
  fprintf('\n\nSKEW_MU_UB_BIS: the LMI solver has failed (4)!\n\n');
  muu=Inf;
  H=NaN*ones(nnn);
  D=H;
  G=H;
  return;
end
Dr=D2(1:n2,1:n2);
Di=D2(1:n2,(n2+1):(2*n2));
D2=Dr+sqrt(-1)*Di;
%
if mixed_2
  G2=dec2mat(LMIs,xopt,G2);
  if size(G2,2)~=(2*n2)
    fprintf('\n\nSKEW_MU_UB_BIS: the LMI solver has failed (5)!\n\n');
    muu=Inf;
    H=NaN*ones(nnn);
    D=H;
    G=H;
    return;
  end
  Gr=G2(1:n2,1:n2);
  Gi=G2(1:n2,(n2+1):(2*n2));
  G2=Gr+sqrt(-1)*Gi;
else
  G2=zeros(size(D2));
end
%
G=mdiag(G1/muu,G2);
D=mdiag(D1/muu/muu,D2);
H=mdiag(muu*eye(size(M11(:,:,1))),eye(size(M22(:,:,1))));
