function [puls,tab_delay,tab_phase,tab_gain]=worst_case_margin(sys_M1,sys_M2,blk,puls,improved_delay_margin)
%
% call:  [puls,tab_delay,tab_phase,tab_gain]=worst_case_margin(sys_M1,sys_M2,blk,puls,improved_delay_margin)
%
% Computation of worst-case gain, phase and delay margins (with the small gain theorem)
% in the presence of "classical" model uncertainties (i.e. parametric
% uncertainties and neglected dynamics). The issue is to compute an estimate
% of the worst-case value of the MIMO gain, phase and delay margins
% when classical model uncertainties are maintained inside the unit ball.
%
% Uncertain gains, phases and delays are represented by additional direct
% or inverse neglected dynamics (typically at the plant inputs and/or outputs).
%
% Use of 2 standard interconnection structures M(s) - diag(Delta,Delta_c):
% Delta contains classical model uncertainties, whose structure is described by blk.
% Delta_c contains additional direct or inverse neglected dynamics
% (whose structure is not described by blk). Note that these neglected dynamics thus
% correspond to the last inputs and outputs of M(s).
%
% sys_M1 corresponds to the standard interconnection structure with direct neglected dynamics.
% sys_M2 corresponds to the standard interconnection structure with inverse ones.
% It is possible to choose sys_M1=[] or sys_M2=[].
%
% If a direct block Delta_c of neglected dynamics is used, I+Delta_c must
% correspond to the uncertain gain k, phase e^(j phi) or delay e^(-tau s).
% More precisely, if e.g. uncertain gains are considered:
% I+Delta_c = diag(k_i) for i between 1 and N.
% The same is true for uncertain phases and delays.
%
% If an inverse block Delta_c of neglected dynamics is used, (I-Delta_c)^(-1) must
% correspond to the uncertain gain k, phase e^(j phi) or delay e^(-tau s).
%
% blk can have 2 or 3 columns. But the value of the 3th column is not accounted for,
% since "classical" model uncertainties are maintained inside the unit ball.
%
% blk=[] means that no classical model uncertainty is accounted for in the problem,
% just additional neglected dynamics representing the uncertain gains, phases or delays.
%
% M(s) and diag(Delta,Delta_c) must be square.
% 
% Basic computation of worst-case gain, phase and delay margins:
% the phase margin corresponds to [-A, A], the delay margin
% to [-A/omega, A/omega] where omega is the frequency.
% (remember phi = - omega * tau, where phi is the phase and tau the delay)
%
% if improved_delay_margin=1 the delay margin is improved, by considering
% an interval [0, tau_max] instead of [-tau_max tau_max]. 
%
% tab_delay, tab_gain and tab_phase represent conservative estimates of
% the margin (i.e. lower bounds) as a function of frequency puls(i).
%
% The initial frequency gridding puls is refined inside the routine.
%
% If the closed loop is not robustly stable in the presence of classical model uncertainties
% maintained inside their unit ball, gain, phase and delay margins are taken as 0.
%
% Skew Mu Toolbox - G. Ferreres and J.M. Biannic, Onera-DCSD Toulouse, France.
%
if nargin~=5 error('WORST_CASE_MARGIN: 5 input arguments'); end
%
% Preliminary tests
%
if isempty(sys_M1)&isempty(sys_M2)
  error('WORST_CASE_MARGIN: at least sys_M1 or sys_M2 must be specified');
end
%
if ~isempty(blk)
  blk=blk(:,1:2);
  idx=find(blk(:,1)<0);
  if ~isempty(idx)
    qqq=blk(idx,2);
    if any(qqq~=0)
      error('WORST_CASE_MARGIN: full real blocks are not allowed');
    end
  end
  if ~square_delta(blk)
    error('WORST_CASE_MARGIN: Delta must be square');
  end
  nnn=sum(abs(blk(:,1)));
else
  nnn=0;
end
%
number_delays=[];
%
for i=1:2
%
  if i==1
    sys=sys_M1;
  else
    sys=sys_M2;
  end
%
  if ~isempty(sys)
    e=eig(sys);
    if max(real(e))>=0
      error('WORST_CASE_MARGIN: M(s) is not asymptotically stable');
    end
    [a,b,c,d]=ssdata(sys);
    if size(d,1)~=size(d,2)
      error('WORST_CASE_MARGIN: M(s) must be square');
    end
    number_delays=[number_delays size(d,1)-nnn];
  end
%
end
%
if length(number_delays)==2
  if number_delays(1)~=number_delays(2)
    error('WORST_CASE_MARGIN: problem with the size of system matrices sys_M1 and sys_M2');
  end
  number_delays=number_delays(1);
end
%
blkk=[blk zeros(size(blk,1),1); ones(number_delays,3)];
%
if ~isempty(sys_M1)
  tab_alfa1=worst_case_margin_2(sys_M1,puls,blkk);
else
  tab_alfa1=Inf*ones(length(puls),1);
end
%
if ~isempty(sys_M2)
  tab_alfa2=worst_case_margin_2(sys_M2,puls,blkk);
else
  tab_alfa2=Inf*ones(length(puls),1);
end
%
tab_alfa=min(tab_alfa1,tab_alfa2);
[qqq,i]=max(tab_alfa);
%
if (puls(i)~=0)
  puls_bis=logspace2(0.8*puls(i),1.2*puls(i),30); % May be adjusted
  if ~isempty(sys_M1)
    tab_alfa1_bis=worst_case_margin_2(sys_M1,puls_bis,blkk);
  else
    tab_alfa1_bis=Inf*ones(length(puls_bis),1);
  end
  tab_alfa1=[tab_alfa1; tab_alfa1_bis];
  if ~isempty(sys_M2)
    tab_alfa2_bis=worst_case_margin_2(sys_M2,puls_bis,blkk);
  else
    tab_alfa2_bis=Inf*ones(length(puls_bis),1);
  end
  tab_alfa2=[tab_alfa2; tab_alfa2_bis];
  puls=[puls puls_bis];
  [puls,idx]=sort(puls);
  tab_alfa1=tab_alfa1(idx);
  tab_alfa2=tab_alfa2(idx);
end
%
tab_gain1 =[];
tab_phase1=[];
tab_delay1=[];
tab_gain2 =[];
tab_phase2=[];
tab_delay2=[];
for i=1:length(puls)
  [gain,phi_max,delay_max]=worst_case_margin_3(tab_alfa1(i),puls(i),0);
  tab_delay1(i)=delay_max;
  tab_phase1(i)=phi_max;
  tab_gain1(i,:)=gain;  
  [gain,phi_max,delay_max]=worst_case_margin_3(tab_alfa2(i),puls(i),1);
  tab_delay2(i)=delay_max;
  tab_phase2(i)=phi_max;
  tab_gain2(i,:)=gain;  
end
%
tab_phase=max(tab_phase1(:),tab_phase2(:));
tab_delay=max(tab_delay1(:),tab_delay2(:));
tab_gain(:,1)=min(tab_gain1(:,1),tab_gain2(:,1));
tab_gain(:,2)=max(tab_gain1(:,2),tab_gain2(:,2));
%
[qqq,i]=min(tab_phase);
fprintf('\n\n w = %6.3f rad/s --> worst-case phase margin = %6.3f degrees\n',puls(i),qqq);
%
[qqq,i]=min(tab_delay);
fprintf('\n w = %6.3f rad/s --> worst-case delay margin = %6.3e s\n',puls(i),qqq);
%
[qqq,i]=max(tab_gain(:,1));
fprintf('\n w = %6.3f rad/s --> worst-case lower gain margin = %6.3f\n',puls(i),qqq);
%
[qqq,i]=min(tab_gain(:,2));
fprintf('\n w = %6.3f rad/s --> worst-case upper gain margin = %6.3f\n\n',puls(i),qqq);
%
if ~improved_delay_margin return; end
%
% *** IMPROVEMENT OF THE DELAY MARGIN ***
%
fprintf('\n\n IMPROVEMENT OF THE DELAY MARGIN \n\n');
%
if ~isempty(sys_M1)
  tab_M1=freqresp(sys_M1,puls);
else
  tab_M1=[];
end
%
if ~isempty(sys_M2)
  tab_M2=freqresp(sys_M2,puls);
else
  tab_M2=[];
end
%
if isempty(blk)
  nnn=0;
else 
  nnn=sum(abs(blk(:,1)));
end
%
delay_margin_mem=[];
%
fprintf('\n');
while 1
%
% Computation of the new delay margin
%
  [delay_margin,iw]=min(tab_delay);
%
  delay_margin_mem=[delay_margin_mem delay_margin];
  if length(delay_margin_mem)>5
    qqq1=min(delay_margin_mem((end-5):end));
    qqq2=max(delay_margin_mem((end-5):end));
    qqq=(qqq2-qqq1)/qqq2;
    stop=(qqq<=1e-3);         % the threshold can be adjusted
  else
    stop=0;
    qqq=NaN;
  end
%
  inverse=(delay_margin==tab_delay2(iw));
  if inverse
    fprintf('delay margin = %6.3e s at w = %6.3f rad/s (%6.3e percent of increase - inverse neglected dynamics)\n',...
    delay_margin,puls(iw),100*qqq);
  else
    fprintf('delay margin = %6.3e s at w = %6.3f rad/s (%6.3e percent of increase - direct neglected dynamics)\n',...
    delay_margin,puls(iw),100*qqq);
  end
%
  if stop break; end
%
  tau0=0.5*delay_margin;
  qqq=exp(-sqrt(-1)*puls(iw)*tau0);
  Delta0=zeros(nnn+number_delays);
  for j=(nnn+1):(nnn+number_delays)
    if inverse==0
      Delta0(j,j)=qqq-1;
    else
      Delta0(j,j)=1-1/qqq;
    end
  end
  if inverse M=tab_M2(:,:,iw); else M=tab_M1(:,:,iw); end
  qqq=Delta0*M;
  qqq=eye(size(qqq))-qqq;
  qqq=inv(qqq);
  MM=qqq*M;
  muub=skew_mu_ub_bis(MM,blkk,1e-2,1,1e-3);
%
  alfa=1/muub;
  if alfa>=2
    delay_max=Inf;
  else
    phi_max=2*asin(alfa/2);
    if puls(iw)==0 delay_max=Inf; else delay_max=phi_max/puls(iw); end
  end
  qqq=tau0+delay_max;
  if qqq>tab_delay(iw)
    tab_delay(iw)=qqq;
  else
    fprintf('It seems no more possible to increase the delay margin\n');
    break;
  end
  if inverse==0
    tab_delay1(iw)=tab_delay(iw);
  else
    tab_delay2(iw)=tab_delay(iw);
  end
%
end
%
fprintf('\n\n The final value gives the improved delay margin (over the whole freq. domain)\n\n');
%
% ***************************************************
%
function tab_alfa=worst_case_margin_2(sys_M,puls,blkk)
%
% call:  tab_alfa=worst_case_margin_2(sys_M,puls,blkk)
%
% a subroutine of worst_case_margin.m
%
% May 03
%
%
tab_M=freqresp(sys_M,puls);
nw=length(puls);
tab_alfa=zeros(nw,1);
%
fprintf('\n%i: ',nw);
for iw=1:nw
  fprintf('%i.',iw);
  M=tab_M(:,:,iw);
  muub=skew_mu_ub_bis(M,blkk,1e-2,1,1e-3);
  tab_alfa(iw)=muub;
end
fprintf('\n');
%
% ***************************************************
%
function [gain,phi_max,delay_max]=worst_case_margin_3(alfa,w,inverse)
%
% call:  [gain,phi_max,delay_max]=worst_case_margin_3(alfa,w,inverse)
%
% a subroutine of worst_case_margin.m
%
% May 03
%
if isinf(alfa) alfa=0; else alfa=1/alfa; end
%
if alfa>=2
  phi_max=pi;
  delay_max=Inf;
else
  phi_max=2*asin(alfa/2);
  if w==0 delay_max=Inf; else delay_max=phi_max/w; end
end
phi_max=180/pi*phi_max;
% 
if ~inverse
  gain=[1-alfa 1+alfa]; % case of direct neglected dynamics
elseif alfa<1
  gain=[1/(1+alfa) 1/(1-alfa)]; % case of inverse neglected dynamics
else
  gain=[1/(1+alfa) Inf];
end
